@@ -1,7 +1,5 @@
 /*
 
-  $Id$
-
   G N O K I I
 
   A Linux/Unix toolset and driver for the mobile phones.
@@ -26,11 +24,11 @@
   Copyright (C) 1999-2000 Hugh Blemings & Pavel Janik ml.
   Copyright (C) 2000      Jan Derfinak
   Copyright (C) 2001      Jan Kratochvil
-  Copyright (C) 2001-2005 Pawel Kot
+  Copyright (C) 2001-2011 Pawel Kot
   Copyright (C) 2002-2004 BORBELY Zoltan
   Copyright (C) 2005      Bastien Nocera
 
-  Config file (/etc/gnokiirc and ~/.gnokiirc) reader.
+  Config reader.
 
   Modified from code by Tim Potter.
 
@@ -53,7 +51,7 @@ GNOKII_API struct gn_cfg_header *gn_cfg_info;
 static gn_config gn_config_default, gn_config_global;
 
 /* Handy macros */
-#define FREE(x)	do { if (x) free(x); (x) = NULL; } while (0)
+#define FREE(x)	do { free(x); (x) = NULL; } while (0)
 
 /*
  * Function to dump configuration pointed by @config
@@ -231,9 +229,7 @@ GNOKII_API struct gn_cfg_header *gn_cfg_variable_set(struct gn_cfg_header *confi
 		return NULL;
 	}
 
-	hdr = cfg_variable_set(hdr, name, value, overwrite);
-
-	return hdr;
+	return cfg_variable_set(hdr, name, value, overwrite);
 }
 
 /*
@@ -573,11 +569,19 @@ struct gn_cfg_header *cfg_file_read(const char *filename)
 		}
 
 		if (nonempty) {
+			void *aux;
+
 			num_lines++;
 			buf = malloc((hash ? hash : line_end) - line_begin + 1);
 			snprintf(buf, (hash ? hash : line_end) - line_begin + 1, "%s", line_begin);
-			split_lines = realloc(split_lines,
+			aux = realloc(split_lines,
 					(num_lines + 1) * sizeof(char*));
+			if (aux)
+				split_lines = aux;
+			else {
+				free(buf);
+				goto err_alloc;
+			}
 			split_lines[num_lines - 1] = buf;
 		}
 
@@ -587,8 +591,6 @@ struct gn_cfg_header *cfg_file_read(const char *filename)
 		}
 	}
 
-
-	free(lines);
 	if (split_lines == NULL)
 		goto out;
 	split_lines[num_lines] = NULL;
@@ -596,6 +598,9 @@ struct gn_cfg_header *cfg_file_read(const char *filename)
 	/* Finally, load the configuration from the split lines */
 	header = cfg_memory_read((const char **)split_lines);
 
+err_alloc:
+	free(lines);
+
 	/* Free the split_lines */
 	for (i = 0; split_lines[i] != NULL; i++) {
 		free(split_lines[i]);
@@ -606,8 +611,7 @@ struct gn_cfg_header *cfg_file_read(const char *filename)
 
 err_read:
 	fclose(handle);
-	if (lines)
-		free(lines);
+	free(lines);
 
 out:
 	return header;
@@ -729,7 +733,7 @@ static gn_error cfg_psection_load(gn_config *cfg, const char *section, const gn_
 		return GN_ERR_NOPHONE;
 	}
 
-	/* You need to specify at least model and port in the phone section */
+	/* You need to specify at least model, port and connection in the phone section */
 	if (!(val = gn_cfg_get(gn_cfg_info, section, "model"))) {
 		fprintf(stderr, _("You need to define '%s' in the config file.\n"), "model");
 		return GN_ERR_NOMODEL;
@@ -804,6 +808,14 @@ static gn_error cfg_psection_load(gn_config *cfg, const char *section, const gn_
 		cfg->require_dcd = def->require_dcd;
 	}
 
+	if (!(val = gn_cfg_get(gn_cfg_info, section, "set_dtr_rts")))
+		cfg->set_dtr_rts = def->set_dtr_rts;
+	else if (sscanf(val, " %d %c", &cfg->set_dtr_rts, &ch) != 1) {
+		fprintf(stderr, _("Unsupported [%s] %s value \"%s\"\n"), section, "set_dtr_rts", val);
+		fprintf(stderr, _("Assuming: %d\n"), def->set_dtr_rts);
+		cfg->set_dtr_rts = def->set_dtr_rts;
+	}
+
 	if (!(val = gn_cfg_get(gn_cfg_info, section, "smsc_timeout")))
 		cfg->smsc_timeout = def->smsc_timeout;
 	else if (sscanf(val, " %d %c", &cfg->smsc_timeout, &ch) == 1)
@@ -865,6 +877,94 @@ static gn_error cfg_psection_load(gn_config *cfg, const char *section, const gn_
 	return GN_ERR_NONE;
 }
 
+#define SETFLAG(f) \
+if (!gnokii_strcmpsep(#f, val, ',')) {\
+	phone_model.flags |= PM_##f;\
+	dprintf("Flag PM_%s\n", #f);\
+	continue;\
+}
+/* Load phone flags from config or fallback to the static array in misc.c */
+gn_phone_model *gn_cfg_get_phone_model(struct gn_cfg_header *cfg, const char *product_name)
+{
+	struct gn_cfg_header *hdr;
+	char *val, *comma, *end, *section = "flags";
+	int count;
+	static gn_phone_model phone_model = {NULL, NULL, 0};
+	gn_phone_model *found_phone_model;
+	static char model[GN_MODEL_MAX_LENGTH] = "";
+
+	if (phone_model.model)
+		return &phone_model;
+
+	val = gn_cfg_get(gn_cfg_info, section, product_name);
+	if (val) {
+		phone_model.model = model;
+		/* Ignore trailing whitespace (leading whitespace has been stripped when loading the config file) */
+		comma = val;
+		while (*comma && *comma != ',')
+			comma++;
+		end = comma;
+		while (end > val && (isspace(*end) || *end == ','))
+			end--;
+		count = end - val + 1;
+		snprintf(model, GN_MODEL_MAX_LENGTH, "%.*s", count, val);
+
+		/* Check flags */
+		val = comma;
+		for (;;) {
+			while (*val && *val != ',')
+				val++;
+			/* This skips also empty fields */
+			while (*val && (*val == ',' || isspace(*val)))
+				val++;
+			if (!*val)
+				break;
+			SETFLAG(OLD_DEFAULT);
+			SETFLAG(DEFAULT);
+			SETFLAG(DEFAULT_S40_3RD);
+			SETFLAG(CALLERGROUP);
+			SETFLAG(NETMONITOR);
+			SETFLAG(KEYBOARD);
+			SETFLAG(SMS);
+			SETFLAG(CALENDAR);
+			SETFLAG(DTMF);
+			SETFLAG(DATA);
+			SETFLAG(SPEEDDIAL);
+			SETFLAG(EXTPBK);
+			SETFLAG(AUTHENTICATION);
+			SETFLAG(FOLDERS);
+			SETFLAG(FULLPBK);
+			SETFLAG(SMSFILE);
+			SETFLAG(EXTPBK2);
+			SETFLAG(EXTCALENDAR);
+			SETFLAG(XGNOKIIBREAKAGE);
+			/* FIXME? duplicated code from above */
+			comma = val;
+			while (*comma && *comma != ',')
+				comma++;
+			end = comma;
+			while (end > val && (isspace(*end) || *end == ','))
+				end--;
+			count = end - val + 1;
+			dprintf("Unknown flag \"%.*s\"\n", count, val);
+		}
+		return &phone_model;
+	}
+#undef SETFLAG
+
+	found_phone_model = gn_phone_model_get(product_name);
+	if (found_phone_model->model)
+		return found_phone_model;
+
+	/* Give the user some hint on why the product_name wasn't found */
+	hdr = cfg_header_get(cfg, (char *)section);
+	if (!hdr) {
+		fprintf(stderr, _("No %s section in the config file.\n"), section);
+	}
+
+	return &phone_model;
+}
+
 static bool cfg_get_log_target(gn_log_target *t, const char *opt)
 {
 	char *val;
@@ -886,43 +986,193 @@ static bool cfg_get_log_target(gn_log_target *t, const char *opt)
 	return true;
 }
 
-#define MAX_PATH_LEN 200
-GNOKII_API gn_error gn_cfg_read_default()
-{
-	gn_error error;
-	char *homedir;
-	char rcfile[MAX_PATH_LEN];
+#define MAX_PATH_LEN 255
+
+#define CHECK_SIZE()	if (*retval >= size) { \
+	void *aux; \
+	size *= 2; \
+	aux = realloc(config_file_locations, size); \
+	if (aux) \
+		config_file_locations = aux; \
+	else {\
+		free(config_file_locations); \
+		config_file_locations = NULL; \
+		goto out; \
+	} \
+}
+
+/*
+ * get_locations() returns the list of possible config file locations that
+ * may be used on the given system.  It returns the array of paths to the
+ * config files.  @retval parameter denotes how many locations are returned. 
+ * The resulting array needs to be freed afterwards.
+ */ 
 #ifdef WIN32
-	char globalrc[MAX_PATH_LEN];
-	char *systemroot;
+/* Windows version */
+static char **get_locations(int *retval)
+{
+	char **config_file_locations;
+	char *appdata, *homedrive, *homepath, *systemroot; /* env variables */
+	int size = 3; /* default size for config_file_locations */
+	char path[MAX_PATH_LEN];
+	const char *fname = "gnokii.ini";
 
+	*retval = size;
+
+	config_file_locations = calloc(size, sizeof(char *));
+
+	appdata = getenv("APPDATA");
+	homedrive = getenv("HOMEDRIVE");
+	homepath = getenv("HOMEPATH");
 	systemroot = getenv("SYSTEMROOT");
-	strncpy(globalrc, systemroot ? systemroot : "", MAX_PATH_LEN);
-	strncat(globalrc, "\\gnokiirc", MAX_PATH_LEN);
+	/* 1. %APPDATA%\gnokii\gnokii.ini */
+	snprintf(path, MAX_PATH_LEN, "%s\\gnokii\\%s", appdata, fname);
+	config_file_locations[0] = strdup(path);
+	/* old gnokii behaviour */
+	/* 2. %HOMEDRIVE%\%HOMEPATH%\gnokii.ini */
+	snprintf(path, MAX_PATH_LEN, "%s\\%s\\%s", homedrive, homepath, fname);
+	config_file_locations[1] = strdup(path);
+	/* 3. %SYSTEMROOT%\gnokii.ini */
+	snprintf(path, MAX_PATH_LEN, "%s\\%s", systemroot, fname);
+	config_file_locations[2] = strdup(path);
 
-	homedir = getenv("HOMEDRIVE");
-	strncpy(rcfile, homedir ? homedir : "", MAX_PATH_LEN);
-	homedir = getenv("HOMEPATH");
-	strncat(rcfile, homedir ? homedir : "", MAX_PATH_LEN);
-	strncat(rcfile, "\\_gnokiirc", MAX_PATH_LEN);
-#else
-	const char globalrc[] = "/etc/gnokiirc";
+	return config_file_locations;
+}
+#else /* WIN32 */
+#  ifdef __MACH__
+/* Mac OS X version */
+static char **get_locations(int *retval)
+{
+	char **config_file_locations;
+	char *home; /* env variables */
+	int size = 3; /* default size for config_file_locations */
+	char path[MAX_PATH_LEN];
 
-	homedir = getenv("HOME");
-	if (homedir)
-		strncpy(rcfile, homedir, MAX_PATH_LEN);
-	strncat(rcfile, "/.gnokiirc", MAX_PATH_LEN);
-#endif
+	*retval = size;
 
-	/* Try opening .gnokirc from users home directory first */
-	if ((error = gn_cfg_file_read(rcfile)) != GN_ERR_NONE) {
-		fprintf(stderr, _("Couldn't read %s config file.\n"), rcfile);
-		/* It failed so try for /etc/gnokiirc */
-		if ((error == GN_ERR_NOCONFIG) && ((error = gn_cfg_file_read(globalrc)) != GN_ERR_NONE)) {
-			/* That failed too so exit */
-			fprintf(stderr, _("Couldn't read %s config file.\n"), globalrc);
+	config_file_locations = calloc(size, sizeof(char *));
+
+	home = getenv("HOME");
+	/* 1. $HOME/Library/Preferences/gnokii/config */
+	snprintf(path, MAX_PATH_LEN, "%s/Library/Preferences/gnokii/config", home);
+	config_file_locations[0] = strdup(path);
+	/* old gnokii behaviour */
+	/* 2. $HOME/.gnokiirc */
+	snprintf(path, MAX_PATH_LEN, "%s/.gnokiirc", home);
+	config_file_locations[1] = strdup(path);
+	/* 3. /etc/gnokiirc */
+	snprintf(path, MAX_PATH_LEN, "/etc/gnokiirc");
+	config_file_locations[2] = strdup(path);
+
+	return config_file_locations;
+}
+#  else /* !__MACH__ */
+/* freedesktop.org compliancy: http://standards.freedesktop.org/basedir-spec/latest/ar01s03.html */
+#define XDG_CONFIG_HOME	"/.config"	/* $HOME/.config */
+#define XDG_CONFIG_DIRS "/etc/xdg"
+static char **get_locations(int *retval)
+{
+	char **config_file_locations = NULL;
+	char *xdg_config_home, *xdg_config_dirs, *home; /* env variables */
+	char **xdg_config_dir;
+	char *aux, *dirs;
+	int j, i = 0;
+	char path[MAX_PATH_LEN];
+	int size = 8; /* default size for config_file_locations */
+	int xcd_size = 4; /* default size for xdg_config_dir - number of elements */
+	int free_xdg_config_home = 0;
+
+	*retval = 0;
+	/* First, let's get all env variables we will need */
+	home = getenv("HOME");
+
+	xdg_config_home = getenv("XDG_CONFIG_HOME");
+	if (!xdg_config_home) {
+		xdg_config_home = calloc(MAX_PATH_LEN, sizeof(char));
+		free_xdg_config_home = 1;
+		snprintf(xdg_config_home, MAX_PATH_LEN, "%s%s", home, XDG_CONFIG_HOME);
+	}
+
+	aux = getenv("XDG_CONFIG_DIRS");
+	if (aux)
+		xdg_config_dirs = strdup(aux);
+	else
+		xdg_config_dirs = strdup(XDG_CONFIG_DIRS);
+
+	dirs = xdg_config_dirs;
+	/* split out xdg_config_dirs into tokens separated by ':' */
+	xdg_config_dir = calloc(xcd_size, sizeof(char *));
+	while ((aux = strsep(&xdg_config_dirs, ":")) != NULL) {
+		xdg_config_dir[i++] = strdup(aux);
+		if (i >= xcd_size) {
+			void *aux;
+			xcd_size *= 2;
+			aux = realloc(xdg_config_dir, xcd_size);
+			if (aux)
+				xdg_config_dir = aux;
+			else {
+				dprintf("Failed to allocate\n");
+				free(xdg_config_dirs);
+				free(dirs);
+				goto out;
+			}
 		}
 	}
+	free(dirs);
+
+	/* 1. $XDG_CONFIG_HOME/gnokii/config ($HOME/.config) */
+	snprintf(path, MAX_PATH_LEN, "%s/gnokii/config", xdg_config_home);
+
+	config_file_locations = calloc(size, sizeof(char *));
+	config_file_locations[(*retval)++] = strdup(path);
+
+	/* old gnokii behaviour */
+	/* 2. $HOME/.gnokiirc */
+	snprintf(path, MAX_PATH_LEN, "%s/.gnokiirc", home);
+	config_file_locations[(*retval)++] = strdup(path);
+	CHECK_SIZE();
+
+	/* 3. $XDG_CONFIG_DIRS/gnokii/config (/etc/xdg) */
+	for (j = 0; j < i; j++) {
+		snprintf(path, MAX_PATH_LEN, "%s/gnokii/config", xdg_config_dir[j]);
+		config_file_locations[(*retval)++] = strdup(path);
+		CHECK_SIZE();
+		free(xdg_config_dir[j]);
+	}
+	free(xdg_config_dir);
+
+	/* old gnokii behaviour */
+	/* 4. /etc/gnokiirc */
+	snprintf(path, MAX_PATH_LEN, "/etc/gnokiirc");
+	config_file_locations[(*retval)++] = strdup(path);
+
+out:
+	if (free_xdg_config_home)
+		free(xdg_config_home);
+
+	return config_file_locations;
+}
+#  endif /* !__MACH__ */
+#endif /* !WIN32 */
+
+GNOKII_API gn_error gn_cfg_read_default()
+{
+	gn_error error = GN_ERR_FAILED;
+	char **config_file_locations = NULL;
+	int num, i;
+
+	config_file_locations = get_locations(&num);
+
+	for (i = 0; i < num && error != GN_ERR_NONE; i++) {
+		error = gn_cfg_file_read(config_file_locations[i]);
+	}
+	for (i = 0; i < num; i++) {
+		/* If it couldn't read any config show all filenames tried */
+		if (error != GN_ERR_NONE)
+			fprintf(stderr, _("Couldn't read %s config file.\n"), config_file_locations[i]);
+		free(config_file_locations[i]);
+	}
+	free(config_file_locations);
 	return error;
 }
 
@@ -959,10 +1209,6 @@ static gn_error cfg_file_or_memory_read(const char *file, const char **lines)
 
 	if (gn_cfg_info == NULL) {
 		/* this is bad, but the previous was much worse - bozo */
-		if (file)
-			fprintf(stderr, _("Couldn't read %s config file.\n"), file);
-		else
-			fprintf(stderr, _("Couldn't read config.\n"));
 		return GN_ERR_NOCONFIG;
 	}
 	gn_config_default.model[0] = 0;
@@ -973,6 +1219,7 @@ static gn_error cfg_file_or_memory_read(const char *file, const char **lines)
 	gn_config_default.serial_write_usleep = -1;
 	gn_config_default.hardware_handshake = false;
 	gn_config_default.require_dcd = false;
+	gn_config_default.set_dtr_rts = true;
 	gn_config_default.smsc_timeout = -1;
 	gn_config_default.irda_string[0] = 0;
 	gn_config_default.connect_script[0] = 0;
@@ -1020,6 +1267,9 @@ GNOKII_API gn_error gn_cfg_phone_load(const char *iname, struct gn_statemachine
 	char section[256];
 	gn_error error;
 
+	if (!state)
+		return GN_ERR_INTERNALERROR;
+
 	if (iname == NULL || *iname == '\0') {
 		state->config = gn_config_global;
 	} else {
