@@ -158,7 +158,7 @@ int gn_vm_initialise(const char *iname, const char *bindir, int debug_mode, int
 	}
 
 	if (dp_Initialise(PtyRDFD, PtyWRFD) != true) {
-		fprintf (stderr, _("gn_vm_Initialise - dp_Initialise failed!\n"));
+		fprintf (stderr, _("gn_vm_initialise - dp_Initialise failed!\n"));
 		return (false);
 	}
 
@@ -362,7 +362,7 @@ static int VM_PtySetup(const char *bindir)
 	strncat(mgnokiidev, "mgnokiidev", sizeof(mgnokiidev) - strlen(mgnokiidev));
 
 	if (access(mgnokiidev, X_OK) != 0) {
-		fprintf(stderr, _("Cannot access %s, check the bindir in your gnokiirc!\n"), mgnokiidev);
+		fprintf(stderr, _("Cannot access %s, check the bindir in your config file!\n"), mgnokiidev);
 		exit(1);
 	}
 
