@@ -1,6 +1,6 @@
 /*
 
-  $Id: device.c,v 1.45 2008/01/09 13:09:46 ladis Exp $
+  $Id$
 
   G N O K I I
 
@@ -41,7 +41,7 @@
 #include "devices/serial.h"
 #include "devices/tekram.h"
 #include "devices/dku2libusb.h"
-
+#include "devices/socketphonet.h"
 
 GNOKII_API int device_getfd(struct gn_statemachine *state)
 {
@@ -78,6 +78,9 @@ int device_open(const char *file, int with_odd_parity, int with_async,
 	case GN_CT_DKU2LIBUSB:
 		state->device.fd = fbusdku2usb_open(state);
 		break;
+	case GN_CT_SOCKETPHONET:
+		state->device.fd = socketphonet_open(file, with_async, state);
+		break;
 	default:
 		state->device.fd = -1;
 		break;
@@ -110,14 +113,15 @@ void device_close(struct gn_statemachine *state)
 	case GN_CT_DKU2LIBUSB:
 		fbusdku2usb_close(state);
 		break;
+	case GN_CT_SOCKETPHONET:
+		socketphonet_close(state);
+		break;
 	default:
 		break;
 	}
 
-	if (state->device.device_instance) {
-		free(state->device.device_instance);
-		state->device.device_instance = NULL;
-	}
+	free(state->device.device_instance);
+	state->device.device_instance = NULL;
 }
 
 void device_reset(struct gn_statemachine *state)
@@ -127,12 +131,11 @@ void device_reset(struct gn_statemachine *state)
 
 void device_setdtrrts(int dtr, int rts, struct gn_statemachine *state)
 {
-	dprintf("Serial device: setting RTS to %s and DTR to %s\n", rts ? "high" : "low", dtr ? "high" : "low");
-
 	switch (state->device.type) {
 	case GN_CT_DKU2:
 	case GN_CT_Serial:
 	case GN_CT_Infrared:
+		dprintf("Serial device: setting RTS to %s and DTR to %s\n", rts ? "high" : "low", dtr ? "high" : "low");
 		serial_setdtrrts(state->device.fd, dtr, rts, state);
 		break;
 	case GN_CT_Irda:
@@ -140,6 +143,7 @@ void device_setdtrrts(int dtr, int rts, struct gn_statemachine *state)
 	case GN_CT_Tekram:
 	case GN_CT_TCP:
 	case GN_CT_DKU2LIBUSB:
+	case GN_CT_SOCKETPHONET:
 	default:
 		break;
 	}
@@ -147,21 +151,22 @@ void device_setdtrrts(int dtr, int rts, struct gn_statemachine *state)
 
 void device_changespeed(int speed, struct gn_statemachine *state)
 {
-	dprintf("Serial device: setting speed to %d\n", speed);
-
 	switch (state->device.type) {
 	case GN_CT_DKU2:
 	case GN_CT_Serial:
 	case GN_CT_Infrared:
+		dprintf("Serial device: setting speed to %d\n", speed);
 		serial_changespeed(state->device.fd, speed, state);
 		break;
 	case GN_CT_Tekram:
+		dprintf("Serial device: setting speed to %d\n", speed);
 		tekram_changespeed(state->device.fd, speed, state);
 		break;
 	case GN_CT_Irda:
 	case GN_CT_Bluetooth:
 	case GN_CT_TCP:
 	case GN_CT_DKU2LIBUSB:
+	case GN_CT_SOCKETPHONET:
 	default:
 		break;
 	}
@@ -184,6 +189,8 @@ size_t device_read(__ptr_t buf, size_t nbytes, struct gn_statemachine *state)
 		return tcp_read(state->device.fd, buf, nbytes, state);
 	case GN_CT_DKU2LIBUSB:
 		return fbusdku2usb_read(buf, nbytes, state);
+	case GN_CT_SOCKETPHONET:
+		return socketphonet_read(state->device.fd, buf, nbytes, state);
 	default:
 		break;
 	}
@@ -207,6 +214,8 @@ size_t device_write(const __ptr_t buf, size_t n, struct gn_statemachine *state)
 		return tcp_write(state->device.fd, buf, n, state);
 	case GN_CT_DKU2LIBUSB:
 		return fbusdku2usb_write(buf, n, state);
+	case GN_CT_SOCKETPHONET:
+		return socketphonet_write(state->device.fd, buf, n, state);
 	default:
 		break;
 	}
@@ -230,6 +239,8 @@ int device_select(struct timeval *timeout, struct gn_statemachine *state)
 		return tcp_select(state->device.fd, timeout, state);
 	case GN_CT_DKU2LIBUSB:
 		return fbusdku2usb_select(timeout, state);
+	case GN_CT_SOCKETPHONET:
+		return socketphonet_select(state->device.fd, timeout, state);
 	default:
 		break;
 	}
