@@ -1,6 +1,6 @@
 /*
 
-  $Id: gsm-bitmaps.c,v 1.45 2007/12/04 19:27:32 hadess Exp $
+  $Id$
 
   G N O K I I
 
@@ -105,7 +105,7 @@ GNOKII_API void gn_bmp_point_clear(gn_bmp *bmp, int x, int y)
 	}
 }
 
-GNOKII_API bool gn_bmp_point(gn_bmp *bmp, int x, int y)
+GNOKII_API int gn_bmp_point(gn_bmp *bmp, int x, int y)
 {
 	int i = 0;
 
