@@ -1,7 +1,5 @@
 /*
 
-  $Id$
-
   G N O K I I
 
   A Linux/Unix toolset and driver for the mobile phones.
@@ -42,6 +40,8 @@ gn_error unimplemented(void)
 
 GNOKII_API gn_memory_type gn_str2memory_type(const char *s)
 {
+	if (!s)
+		return GN_MT_XX;
 #define X(a) if (!strcmp(s, #a)) return GN_MT_##a;
 	X(ME);
 	X(SM);
@@ -55,6 +55,8 @@ GNOKII_API gn_memory_type gn_str2memory_type(const char *s)
 	X(BD);
 	X(SD);
 	X(MT);
+	X(TA);
+	X(CB);
 	X(IN);
 	X(OU);
 	X(AR);
@@ -82,6 +84,7 @@ GNOKII_API gn_memory_type gn_str2memory_type(const char *s)
 	X(F18);
 	X(F19);
 	X(F20);
+	X(BM);
 	return GN_MT_XX;
 #undef X
 }
@@ -104,6 +107,8 @@ GNOKII_API const char *gn_memory_type2str(gn_memory_type mt)
 	X(BD);
 	X(SD);
 	X(MT);
+	X(TA);
+	X(CB);
 	X(IN);
 	X(OU);
 	X(AR);
@@ -131,6 +136,7 @@ GNOKII_API const char *gn_memory_type2str(gn_memory_type mt)
 	X(F18);
 	X(F19);
 	X(F20);
+	X(BM);
 	default: return NULL;
 	}
 #undef X
@@ -144,12 +150,12 @@ GNOKII_API const char *gn_memory_type_print(gn_memory_type mt)
 	case GN_MT_FD: return _("Fixed dial numbers");
 	case GN_MT_ON: return _("Own numbers");
 	case GN_MT_EN: return _("Emergency numbers");
-	case GN_MT_DC: return _("Dialed numbers");
+	case GN_MT_DC: return _("Dialled numbers");
 	case GN_MT_RC: return _("Received numbers");
 	case GN_MT_MC: return _("Missed numbers");
-	case GN_MT_LD: return _("Last dialed");
-	case GN_MT_BD: return _("Barred dialing numbers");
-	case GN_MT_SD: return _("Service dialing numbers");
+	case GN_MT_LD: return _("Last dialled");
+	case GN_MT_BD: return _("Barred dialling numbers");
+	case GN_MT_SD: return _("Service dialling numbers");
 	case GN_MT_MT: return _("Combined ME and SIM phonebook");
 	case GN_MT_TA: return _("Computer memory");
 	case GN_MT_CB: return _("Currently selected memory");
@@ -180,6 +186,7 @@ GNOKII_API const char *gn_memory_type_print(gn_memory_type mt)
 	case GN_MT_F18: return _("SMS Folder 18");
 	case GN_MT_F19: return _("SMS Folder 19");
 	case GN_MT_F20: return _("SMS Folder 20");
+	case GN_MT_BM: return _("Cell Broadcast");
 	default: return _("Unknown");
 	}
 }
@@ -199,6 +206,8 @@ GNOKII_API const char *gn_connection_type2str(gn_connection_type t)
 	case GN_CT_M2BUS:	return _("M2BUS");
 	case GN_CT_DKU2:	return _("DKU2 (kernel support)");
 	case GN_CT_DKU2LIBUSB:	return _("DKU2 (libusb support)");
+	case GN_CT_PCSC:	return _("Smart Card reader (libpcsclite support)");
+	case GN_CT_SOCKETPHONET:	return _("Linux Phonet (kernel support)");
 	default:		return _("Unknown");
 	}
 }
@@ -277,6 +286,7 @@ GNOKII_API const char *gn_phonebook_entry_type2str(gn_phonebook_entry_type t)
 	case GN_PHONEBOOK_ENTRY_Company:		return _("Company");
 	case GN_PHONEBOOK_ENTRY_Nickname:		return _("Nickname");
 	case GN_PHONEBOOK_ENTRY_Birthday:		return _("Birthday");
+	case GN_PHONEBOOK_ENTRY_ExtGroup:		return _("Caller group id");
 	default:					return _("Unknown");
 	}
 }
@@ -314,6 +324,7 @@ GNOKII_API const char *gn_subentrytype2string(gn_phonebook_entry_type entry_type
 	case GN_PHONEBOOK_ENTRY_Company:
 	case GN_PHONEBOOK_ENTRY_Nickname:
 	case GN_PHONEBOOK_ENTRY_Birthday:
+	case GN_PHONEBOOK_ENTRY_ExtGroup:
 		return gn_phonebook_entry_type2str(entry_type);
 	case GN_PHONEBOOK_ENTRY_Number:
 		switch (number_type) {
