@@ -1,6 +1,6 @@
 /*
 
-  $Id: gsm-error.c,v 1.24 2006/06/11 16:39:48 deller Exp $
+  $Id$
 
   G N O K I I
 
@@ -45,7 +45,7 @@ GNOKII_API char *gn_error_print(gn_error e)
 	case GN_ERR_INVALIDSECURITYCODE:   return _("Invalid Security code.");
 	case GN_ERR_INTERNALERROR:         return _("Problem occurred internal to model specific code.");
 	case GN_ERR_NOTIMPLEMENTED:        return _("Command called isn't implemented in model.");
-	case GN_ERR_NOTSUPPORTED:          return _("Function or connection type not supported by the phone.");
+	case GN_ERR_NOTSUPPORTED:          return _("Function or connection type not supported by the phone or by the phone driver.");
 	case GN_ERR_USERCANCELED:          return _("User aborted the action.");
 	case GN_ERR_UNKNOWN:               return _("Unknown error - well better than nothing!!");
 	case GN_ERR_MEMORYFULL:            return _("The specified memory is full.");
@@ -68,6 +68,15 @@ GNOKII_API char *gn_error_print(gn_error e)
 	case GN_ERR_NONEWCBRECEIVED:       return _("Attempt to read CB when no new CB received.");
 	case GN_ERR_SIMPROBLEM:            return _("SIM card missing or damaged.");
 	case GN_ERR_CODEREQUIRED:          return _("PIN or PUK code required.");
+	case GN_ERR_NOTAVAILABLE:          return _("The requested information is not available.");
+	case GN_ERR_NOCONFIG:              return _("Config file cannot be read.");
+	case GN_ERR_NOPHONE:               return _("Either global or given phone section cannot be found.");
+	case GN_ERR_NOLOG:                 return _("Incorrect logging section configuration.");
+	case GN_ERR_NOMODEL:               return _("No phone model specified in the config file.");
+	case GN_ERR_NOPORT:                return _("No port specified in the config file.");
+	case GN_ERR_NOCONNECTION:          return _("No connection type specified in the config file.");
+	case GN_ERR_LOCKED:                return _("Device is locked and cannot be unlocked.");
+	case GN_ERR_ASYNC:                 return _("The actual response will be sent asynchronously.");
 	default:                           return _("Unknown error.");
 	}
 }
