@@ -1,7 +1,5 @@
 /*
 
-  $Id$
-
   G N O K I I
 
   A Linux/Unix toolset and driver for the mobile phones.
@@ -22,7 +20,7 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 2001-2003 Pawel Kot
+  Copyright (C) 2001-2011 Pawel Kot
   Copyright (C) 2001-2002 Markus Plail, Pavel Machek <pavel@ucw.cz>
   Copyright (C) 2002-2003 Ladis Michl
   Copyright (C) 2003-2004 BORBELY Zoltan
@@ -34,6 +32,8 @@
 
 #include "config.h"
 
+#include <glib.h>
+
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
@@ -235,8 +235,8 @@ gn_timestamp *sms_timestamp_unpack(unsigned char *number, gn_timestamp *dt)
 
 /**
  * sms_timestamp_pack - converts gnokii's datetime struct to the binary datetime
- * @number: binary variable to be filled
  * @dt: datetime structure to be read
+ * @number: binary variable to be filled
  *
  */
 unsigned char *sms_timestamp_pack(gn_timestamp *dt, unsigned char *number)
@@ -287,7 +287,7 @@ unsigned char *sms_timestamp_pack(gn_timestamp *dt, unsigned char *number)
  * received SMS - delivery report and fills in the SMS structure with
  * the correct message according to the GSM specification.
  * This function only applies to the delivery reports. Delivery reports
- * contain only one part, so it is assumed all other part except 0th are
+ * contain only one part, so it is assumed all other parts except 0th are
  * NULL.
  */
 static gn_error sms_status(unsigned char status, gn_sms *sms)
@@ -372,7 +372,7 @@ static gn_error sms_status(unsigned char status, gn_sms *sms)
 				dprintf("SM does not exist");
 				break;
 			default:
-				dprintf("Reserved/Specific to SC: %x", status);
+				dprintf("Reserved/Specific to SC: 0x%02x", status);
 				break;
 			}
 		}
@@ -401,7 +401,7 @@ static gn_error sms_status(unsigned char status, gn_sms *sms)
 			dprintf("Error in SME");
 			break;
 		default:
-			dprintf("Reserved/Specific to SC: %x", status);
+			dprintf("Reserved/Specific to SC: 0x%02x", status);
 			break;
 		}
 	} else {
@@ -409,7 +409,7 @@ static gn_error sms_status(unsigned char status, gn_sms *sms)
 		snprintf(sms->user_data[0].u.text, sizeof(sms->user_data[0].u.text), "%s", _("Unknown"));
 
 		/* more detailed reason only for debug */
-		dprintf("Reserved/Specific to SC: %x", status);
+		dprintf("Reserved/Specific to SC: 0x%02x", status);
 	}
 	dprintf("\n");
 	sms->user_data[0].length = strlen(sms->user_data[0].u.text);
@@ -432,9 +432,18 @@ static gn_error sms_data_decode(unsigned char *message, unsigned char *output, u
 				 unsigned int size, unsigned int udhlen, gn_sms_dcs dcs)
 {
 	/* Unicode */
+	if (dcs.type & 0x20) {
+		dprintf("Compressed message\n");
+		return GN_ERR_NOTIMPLEMENTED;
+	}
 	if ((dcs.type & 0x08) == 0x08) {
 		dprintf("Unicode message\n");
-		char_unicode_decode(output, message, length);
+		/*
+		 * length is rawsms->length which is number of characters in
+		 * the decoded text. 3rd argument of char_unicode_decode is
+		 * number of bytes.
+		 */
+		char_unicode_decode(output, message, 2 * length);
 	} else {
 		/* 8bit SMS */
 		if ((dcs.type & 0xf4) == 0xf4) {
@@ -563,7 +572,7 @@ static gn_error sms_udh_decode(unsigned char *message, gn_sms_udh *udh)
 		case 0x07: /* UDH Source Indicator */
 		default:
 			udh->udh[nr].type = GN_SMS_UDH_Unknown;
-			dprintf("Not supported UDH\n");
+			dprintf("User Data Header type 0x%02x isn't supported\n", message[pos]);
 			break;
 		}
 		length -= (udh_length + 2);
@@ -616,7 +625,7 @@ static gn_error sms_header_decode(gn_sms_raw *rawsms, gn_sms *sms, gn_sms_udh *u
 		dprintf("Mobile Originated (sent) message:\n");
 		break;
 	default:
-		dprintf("Not supported message type: %d\n", sms->type);
+		dprintf("SMS message type 0x%02x isn't supported\n", sms->type);
 		return GN_ERR_NOTSUPPORTED;
 	}
 
@@ -634,7 +643,7 @@ static gn_error sms_header_decode(gn_sms_raw *rawsms, gn_sms *sms, gn_sms_udh *u
 	dprintf("\tSMS center number: %s\n", sms->smsc.number);
 
 	/* Delivery time */
-	if (sms->type == GN_SMS_MT_DeliveryReport) {
+	if (sms->type == GN_SMS_MT_StatusReport) {
 		sms_timestamp_unpack(rawsms->time, &(sms->time));
 		dprintf("\tDelivery date: %s\n", sms_timestamp_print(rawsms->time));
 	}
@@ -836,7 +845,7 @@ gn_error gn_sms_pdu2raw(gn_sms_raw *rawsms, unsigned char *pdu, int pdu_len, int
 		dprintf("SMS-DELIVER found\n");
 		/* TP-MMS  TP-More-Messages-to-Send */
 		rawsms->more_messages       = pdu[offset] & 0x04;
-		// bits 3 and 4 of the first octet unused;
+		/* Bits 3 and 4 of the first octet are unused */
 		/* TP-SRI  TP-Status-Report-Indication */
 		rawsms->report_status       = pdu[offset] & 0x20;
 		/* TP-UDHI TP-User-Data-Header-Indication */
@@ -983,6 +992,11 @@ gn_error gn_sms_pdu2raw(gn_sms_raw *rawsms, unsigned char *pdu, int pdu_len, int
 		return GN_ERR_INTERNALERROR;
 	}
 
+	/*
+	 * Unicode message has length in octets. We need length in characters: see comment in sms_pdu_decode().
+	 */
+	if ((rawsms->dcs & 0x08) == 0x08)
+		rawsms->length /= 2;
 out:
 	return ret;
 #undef COPY_USER_DATA
@@ -1164,10 +1178,9 @@ static gn_error sms_get_deleted(gn_data *data)
 
 static gn_error sms_verify_status(gn_data *data)
 {
-	int i, j, found = 0;
+	int i, j;
 
 	for (i = 0; i < data->folder_stats[data->sms_folder->folder_id]->used; i++) {		/* Cycle through all messages we know of */
-		found = 0;
 		if ((data->message_list[i][data->sms_folder->folder_id]->status == GN_SMS_FLD_NotRead) ||	/* if it is a unread one, i.e. not in folderstatus */
 				(data->message_list[i][data->sms_folder->folder_id]->status == GN_SMS_FLD_NotReadHandled)) {
 			for (j = 0; j < data->sms_folder->number; j++) {
@@ -1293,7 +1306,7 @@ static char *sms_udh_encode(gn_sms_raw *rawsms, int type)
 		rawsms->length += headers[type].length;
 		break;
 	default:
-		dprintf("Not supported User Data Header type\n");
+		dprintf("User Data Header type 0x%02x isn't supported\n", type);
 		break;
 	}
 	if (!rawsms->udh_indicator) {
@@ -1307,7 +1320,7 @@ static char *sms_udh_encode(gn_sms_raw *rawsms, int type)
 /**
  * sms_concat_header_encode - Adds concatenated messages header
  * @rawsms: processed SMS
- * @this: current part number
+ * @curr: current part number
  * @total: total parts number
  *
  * This function adds sequent part of the concatenated messages header. Note
@@ -1324,16 +1337,14 @@ static gn_error sms_concat_header_encode(gn_sms_raw *rawsms, int curr, int total
 }
 
 /**
- * sms_data_encode - encodes the date from the SMS structure to the phone frame
+ * sms_data_encode - Encodes the data from the SMS structure to the phone frame
  *
- * @SMS: SMS structure to be encoded
- * @dcs: Data Coding Scheme field in the frame to be set
- * @message: phone frame to be filled in
- * @multipart: do we send one message or more?
- * @clen: coded data length
+ * @sms: #gn_sms structure holding user provided data
+ * @rawsms: #gn_sms_raw structure to be filled with encoded data
  *
- * This function does the phone frame encoding basing on the given SMS
- * structure. This function is capable of creating only one frame at a time.
+ * This function converts values from libgnokii format to a format more similar
+ * to the GSM standard.
+ * This function is capable of creating only one frame at a time.
  */
 static gn_error sms_data_encode(gn_sms *sms, gn_sms_raw *rawsms)
 {
@@ -1351,10 +1362,11 @@ static gn_error sms_data_encode(gn_sms *sms, gn_sms_raw *rawsms)
 		case 2: rawsms->dcs |= 0xf1; break; /* Class 1 */
 		case 3: rawsms->dcs |= 0xf2; break; /* Class 2 */
 		case 4: rawsms->dcs |= 0xf3; break; /* Class 3 */
-		default: dprintf("What ninja-mutant class is this?\n"); break;
+		default: dprintf("General Data Coding class 0x%02x isn't supported\n", sms->dcs.u.general.m_class); break;
 		}
 		if (sms->dcs.u.general.compressed) {
 			/* Compression not supported yet */
+			dprintf("SMS message compression isn't supported\n");
 			/* dcs[0] |= 0x20; */
 		}
 		al = sms->dcs.u.general.alphabet;
@@ -1370,6 +1382,7 @@ static gn_error sms_data_encode(gn_sms *sms, gn_sms_raw *rawsms)
 
 		break;
 	default:
+		dprintf("Data Coding Scheme type 0x%02x isn't supported\n", sms->dcs.type);
 		return GN_ERR_WRONGDATAFORMAT;
 	}
 
@@ -1419,7 +1432,7 @@ static gn_error sms_data_encode(gn_sms *sms, gn_sms_raw *rawsms)
 		case GN_SMS_DATA_Text: {
 			unsigned int length, udh_length, offset = rawsms->user_data_length;
 
-			length = strlen(sms->user_data[i].u.text);
+			length = sms->user_data[i].length;
 			if (sms->udh.length)
 				udh_length = sms->udh.length + 1;
 			else
@@ -1433,6 +1446,8 @@ static gn_error sms_data_encode(gn_sms *sms, gn_sms_raw *rawsms)
 						      &length);
 				rawsms->length = length + (udh_length * 8 + 6) / 7;
 				rawsms->user_data_length = size + offset;
+				dprintf("\tsize: %d\n\toffset: %d\n", size, offset);
+				dprintf("\tencoded size: %d\n\trawsms length: %d\n\trawsms user data length: %d\n", size, rawsms->length, rawsms->user_data_length);
 				break;
 			case GN_SMS_DCS_8bit:
 				dprintf("8bit\n");
@@ -1443,7 +1458,11 @@ static gn_error sms_data_encode(gn_sms *sms, gn_sms_raw *rawsms)
 			case GN_SMS_DCS_UCS2:
 				dprintf("UCS-2\n");
 				rawsms->dcs |= 0x08;
-				length = char_unicode_encode(rawsms->user_data + offset, sms->user_data[i].u.text, length);
+				length = ucs2_encode(rawsms->user_data + offset, GN_SMS_LONG_MAX_LENGTH, sms->user_data[i].u.text, length);
+				if (length < 0) {
+					dprintf("Failed to encode UCS2 message\n");
+					return GN_ERR_WRONGDATAFORMAT;
+				}
 				rawsms->user_data_length = rawsms->length = length + udh_length;
 				break;
 			default:
@@ -1507,7 +1526,7 @@ static gn_error sms_data_encode(gn_sms *sms, gn_sms_raw *rawsms)
 			return GN_ERR_NONE;
 
 		default:
-			dprintf("What kind of ninja-mutant user_data is this?\n");
+			dprintf("User Data type 0x%02x isn't supported\n", sms->user_data[i].type);
 			break;
 		}
 	}
@@ -1525,7 +1544,7 @@ gn_error sms_prepare(gn_sms *sms, gn_sms_raw *rawsms)
 		break;
 	case GN_SMS_MT_DeliveryReport:
 	default:
-		dprintf("Not supported message type: %d\n", rawsms->type);
+		dprintf("Raw SMS message type 0x%02x isn't supported\n", rawsms->type);
 		return GN_ERR_NOTSUPPORTED;
 	}
 	/* Encoding the header */
@@ -1536,7 +1555,7 @@ gn_error sms_prepare(gn_sms *sms, gn_sms_raw *rawsms)
 		return GN_ERR_ENTRYTOOLONG;
 	}
 	rawsms->validity_indicator = GN_SMS_VP_RelativeFormat;
-	rawsms->validity[0] = 0xa9;
+	rawsms->validity[0] = GN_SMS_VP_72H;
 
 	for (i = 0; i < sms->udh.number; i++)
 		if (sms->udh.udh[i].type == GN_SMS_UDH_ConcatenatedMessages)
@@ -1553,16 +1572,26 @@ static void sms_dump_raw(gn_sms_raw *rawsms)
 
 	memset(buf, 0, 10240);
 
-	dprintf("dcs: 0x%x\n", rawsms->dcs);
-	dprintf("Length: 0x%x\n", rawsms->length);
-	dprintf("user_data_length: 0x%x\n", rawsms->user_data_length);
+	dprintf("dcs: 0x%02x\n", rawsms->dcs);
+	dprintf("Length: 0x%02x\n", rawsms->length);
+	dprintf("user_data_length: 0x%02x\n", rawsms->user_data_length);
 	dprintf("ValidityIndicator: %d\n", rawsms->validity_indicator);
 	bin2hex(buf, rawsms->user_data, rawsms->user_data_length);
 	dprintf("user_data: %s\n", buf);
 #endif
 }
 
-static gn_error sms_send_long(gn_data *data, struct gn_statemachine *state);
+
+GNOKII_API int gn_sms_udh_add(gn_sms *sms, gn_sms_udh_type type)
+{
+	sms->udh.length += headers[type].length;
+	sms->udh.udh[sms->udh.number].type = type;
+	sms->udh.number++;
+	return sms->udh.number - 1;
+}
+
+static gn_error sms_send_single(gn_data *data, struct gn_statemachine *state);
+static gn_error sms_send_long(gn_data *data, struct gn_statemachine *state, int octets);
 
 /**
  * gn_sms_send - The main function for the SMS sending
@@ -1573,170 +1602,243 @@ static gn_error sms_send_long(gn_data *data, struct gn_statemachine *state);
  * (gn_sms) in the higher level. This is converted to raw_sms here,
  * and then phone driver takes the fields it needs and sends it in the
  * phone specific way to the phone.
+ *
+ * In case of multipart message, sending is interrupted on the first
+ * failure.  It means that if message got split into 3 messages, and 2nd
+ * part fails to send, the 3rd part is not attepted to get sent.  There is
+ * no interface to retry multipart message from the failed part.
+ *
+ * data->sms->parts indicates how many parts were (or supposed to be) sent.
+ *
+ * data->sms->reference contains data->sms->parts reference numbers from the
+ * sent messages.  They might be used to relate the delivery reports.  If
+ * given element has reference 0, sending this part, falied.
  */
 GNOKII_API gn_error gn_sms_send(gn_data *data, struct gn_statemachine *state)
 {
-	int i;
-	gn_error error = GN_ERR_NONE;
-	/*
-	 * This is for long sms handling. There we have sequence:
-	 * 1.		gn_sms_send()
-	 * 1.1			rawsms = malloc()
-	 * 1.2			sms_send_long()
-	 * 1.2.1			gn_sms_send()
-	 * 1.2.1.1				rawsms = malloc() // original rawsms from 1.1 is lost here
-	 * 1.2.1.2				free(rawsms)
-	 * 1.2.2			gn_sms_send()
-	 * 1.2.2.1				rawsms = malloc()
-	 * 1.2.2.2				free(rawsms)
-	 * ...
-	 * 1.2.N			gn_sms_send()
-	 * 1.2.N.1				rawsms = malloc()
-	 * 1.2.N.2				free(rawsms)
-	 * 1.3			free(rawsms) // rawsms was freed here just above
-	 * Therefore we store old rawsms value on the entrance and restore it on exit.
-	 */
-	gn_sms_raw *old = data->raw_sms;
-
-	if (!data->sms)
-		return GN_ERR_INTERNALERROR;
-
-	data->raw_sms = malloc(sizeof(*data->raw_sms));
-	memset(data->raw_sms, 0, sizeof(*data->raw_sms));
-
-	data->raw_sms->status = GN_SMS_Sent;
-
-	data->raw_sms->message_center[0] = char_semi_octet_pack(data->sms->smsc.number, data->raw_sms->message_center + 1, data->sms->smsc.type);
-	if (data->raw_sms->message_center[0] % 2) data->raw_sms->message_center[0]++;
-	if (data->raw_sms->message_center[0])
-		data->raw_sms->message_center[0] = data->raw_sms->message_center[0] / 2 + 1;
-
-	error = sms_prepare(data->sms, data->raw_sms);
-	if (error != GN_ERR_NONE)
-		goto cleanup;
-
-	sms_dump_raw(data->raw_sms);
-	if (data->raw_sms->user_data_length > MAX_SMS_PART) {
-		dprintf("SMS is too long? %d\n", data->raw_sms->user_data_length);
-		error = sms_send_long(data, state);
-		goto cleanup;
-	}
-
-	dprintf("Sending\n");
-	error = gn_sm_functions(GN_OP_SendSMS, data, state);
-	if (data->sms->parts == 0) {
-		data->sms->parts = 1;
-		/* This is not multipart SMS. For multipart reference is allocated in sms_send_long() */
-		data->sms->reference = calloc(1, sizeof(unsigned int));
-	}
-	
-	/* We send SMS parts from the first part to last. */
-	i = 0;
-	while (i < data->sms->parts-1 && data->sms->reference[i] != 0)
-		i++;
-	data->sms->reference[i] = data->raw_sms->reference;
-
-cleanup:
-	if (data->raw_sms)
-		free(data->raw_sms);
-	data->raw_sms = old;
-	return error;
-}
-
-GNOKII_API int gn_sms_udh_add(gn_sms *sms, gn_sms_udh_type type)
-{
-	sms->udh.length += headers[type].length;
-	sms->udh.udh[sms->udh.number].type = type;
-	sms->udh.number++;
-	return sms->udh.number - 1;
-}
-
-static gn_error sms_send_long(gn_data *data, struct gn_statemachine *state)
-{
-	static int init = 0;
-	int i, j, k, size, count, start, copied, total, refnum, isConcat = -1, max_sms_len = MAX_SMS_PART;
+	int i, total;
+	int enc_chars, ext_chars;
+	gn_error retval;
 	gn_sms sms;
-	gn_sms_user_data ud[GN_SMS_PART_MAX_NUMBER];
-	gn_error error = GN_ERR_NONE;
+	gn_sms *orig_sms;
 
-	/* count -- number of SMS to be sent
-	 * total -- total number of octets to be sent
+	dprintf("=====> ENTER gn_sms_send()\n");
+	/*
+	 * Make sure that we will not get any corruption if it was not
+	 * initialized properly by the app.  Issue a warning that
+	 * application may leak the memory
 	 */
-	sms = *data->sms;
+	if (data->sms->reference) {
+		dprintf("data->sms->reference was not set to NULL. The app may not initialize it\nproperly or leak memory.\n");
+		data->sms->reference = NULL;
+	}
 
-	/* If there's no concat header we need to add one */
-	for (i = 0; i < data->sms->number; i++)
-		if (data->sms->udh.udh[i].type == GN_SMS_UDH_ConcatenatedMessages)
-			isConcat = i;
+	/*
+	 * We need to work on data->sms->user_data; let's have a copy of the
+	 * original request so message could be resubmitted.
+	 * FIXME: use for that only additional user_data structure
+	 */
+	orig_sms = data->sms;
+	memcpy(&sms, data->sms, sizeof(gn_sms));
+	data->sms = &sms;
 
-	if (isConcat == -1)
-		isConcat = gn_sms_udh_add(data->sms, GN_SMS_UDH_ConcatenatedMessages);
+	/* Convert all the input strings to UTF-8 */
+	if (data->sms->dcs.u.general.alphabet != GN_SMS_DCS_8bit) {
+		i = 0;
+		data->sms->dcs.u.general.alphabet = GN_SMS_DCS_DefaultAlphabet;
+		while (data->sms->user_data[i].type != GN_SMS_DATA_None) {
+			gchar *str;
+			gsize inlen, outlen;
+			gn_sms_dcs_alphabet_type enc;
 
+			if (data->sms->user_data[i].type == GN_SMS_DATA_Text ||
+			    data->sms->user_data[i].type == GN_SMS_DATA_NokiaText) {
+			       	str = g_locale_to_utf8(data->sms->user_data[i].u.text, -1, &inlen, &outlen, NULL);
+			       	data->sms->user_data[i].chars = g_utf8_strlen(str, outlen);
+			       	memset(data->sms->user_data[i].u.text, 0, sizeof(data->sms->user_data[i].u.text));
+			       	g_utf8_strncpy(data->sms->user_data[i].u.text, str, data->sms->user_data[i].chars);
+			       	g_free(str);
+				/* Let's make sure the encoding is correct */
+				enc = char_def_alphabet_string_stats(data->sms->user_data[i].u.text, &enc_chars, &ext_chars);
+				if (enc == GN_SMS_DCS_UCS2)
+					data->sms->dcs.u.general.alphabet = enc;
+				data->sms->user_data[i].chars = enc_chars;
+			} else {
+				/* FIXME: that's wrong for Nokia Multipart Picture Message, does anybody still use it? */
+				data->sms->dcs.u.general.alphabet = GN_SMS_DCS_8bit;
+			}
+			i++;
+		}
+	}
+
+	dprintf("enc_chars: %d\next_chars: %d\n", enc_chars, ext_chars);
 	/* Count the total length of the message text octets to be sent */
 	total = 0;
 	i = 0;
 	while (data->sms->user_data[i].type != GN_SMS_DATA_None) {
 		switch (data->sms->dcs.u.general.alphabet) {
 		case GN_SMS_DCS_DefaultAlphabet:
-			total += (data->sms->user_data[i].length * 7 + 7) / 8;
+			/*
+			 * Extended alphabet chars are doubled on the input.
+			 */
+			total += ((enc_chars + ext_chars) * 7 + 7) / 8;
 			break;
 		case GN_SMS_DCS_UCS2:
-			total += (data->sms->user_data[i].length * 2);
+			total += (enc_chars * 2);
 			break;
 		default:
 			total += data->sms->user_data[i].length;
 			break;
 		}
-		memcpy(&ud[i], &data->sms->user_data[i], sizeof(gn_sms_user_data));
 		i++;
 	}
 
+	dprintf("total: %d\n", total);
+
+	/* FIXME: from now on we handle only the user_data[0] */
+	dprintf("size of the input string: %d bytes\n", data->sms->user_data[0].length);
+	dprintf("number of characters in the input string: %d chars\n", data->sms->user_data[0].chars);
+
+	/* It will eventually get overwritten in sms_send_long() */
+	data->sms->parts = 1;
+
+	if (total > MAX_SMS_PART)
+		retval = sms_send_long(data, state, total);
+	else
+		retval = sms_send_single(data, state);
+
+	data->sms = orig_sms;
+	/* Let's put back information about sent messages */
+	data->sms->reference = sms.reference;
+	data->sms->parts = sms.parts;
+
+	return retval;
+}
+
+static gn_error sms_send_single(gn_data *data, struct gn_statemachine *state)
+{
+	int i = 0;
+	gn_error error = GN_ERR_NONE;
+	gn_sms_raw rawsms;
+
+	data->raw_sms = &rawsms;
+	memset(&rawsms, 0, sizeof(rawsms));
+
+	if (!data->sms)
+		return GN_ERR_INTERNALERROR;
+
+	if (data->sms->remote.number[0] == 0) {
+		dprintf("Recipient number cannot be NULL.\n");
+		return GN_ERR_WRONGDATAFORMAT;
+	}
+
+	dprintf("=====> ENTER sms_send_single()\n");
+	data->raw_sms->status = GN_SMS_Sent;
+
+	data->raw_sms->message_center[0] = char_semi_octet_pack(data->sms->smsc.number, data->raw_sms->message_center + 1, data->sms->smsc.type);
+	if (data->raw_sms->message_center[0] % 2)
+		data->raw_sms->message_center[0]++;
+	if (data->raw_sms->message_center[0])
+		data->raw_sms->message_center[0] = data->raw_sms->message_center[0] / 2 + 1;
+
+	error = sms_prepare(data->sms, data->raw_sms);
+	if (error != GN_ERR_NONE)
+		return error;
+
+	sms_dump_raw(data->raw_sms);
+	dprintf("Input is %d bytes long\n", data->sms->user_data[0].length);
+	dprintf("SMS is %d octets long\n", data->raw_sms->user_data_length);
+
+	dprintf("Sending\n");
+	error = gn_sm_functions(GN_OP_SendSMS, data, state);
+
+	/* If there was an error, let's not put the reference into data->sms */
+	if (error == GN_ERR_NONE) {
+		/* In case of multipart message it is already allocated */
+		if (!data->sms->reference)
+			data->sms->reference = calloc(data->sms->parts, sizeof(unsigned int));
+
+		/* We send SMS parts from the first part to last. */
+		i = 0;
+		while (i < data->sms->parts-1 && data->sms->reference[i] != 0)
+			i++;
+		data->sms->reference[i] = data->raw_sms->reference;
+	}
+
+	return error;
+}
+
+/* 
+ * FIXME: we stop sending multipart message on first error. We may end up
+ * with few messages send, and few failed.  We're not able to resend only
+ * failed ones.
+ */
+static gn_error sms_send_long(gn_data *data, struct gn_statemachine *state, int octets)
+{
+	int i, j, k, count, size, start, copied, refnum, is_concat = -1, max_sms_len = MAX_SMS_PART;
+	gn_sms_user_data ud[GN_SMS_PART_MAX_NUMBER];
+	gn_error error = GN_ERR_NONE;
+	time_t t;
+
+	start = 0;
+	copied = 0;
+	count = data->sms->parts;
+
+	dprintf("=====> ENTER sms_send_long()\n");
+
+	dprintf("count: %d\n", count);
+
+	/* If there's no concat header we need to add one */
+	for (i = 0; i < data->sms->number; i++) {
+		if (data->sms->udh.udh[i].type == GN_SMS_UDH_ConcatenatedMessages)
+			is_concat = i;
+	}
+	if (is_concat == -1)
+		is_concat = gn_sms_udh_add(data->sms, GN_SMS_UDH_ConcatenatedMessages);
+
 	/* We need to attach user data header to each part */
 	max_sms_len -= (data->sms->udh.length + 1);
-	dprintf("max_sms_len: %d\n", max_sms_len);
+
 	/* Count number of SMS to be sent */
-	count = (total + max_sms_len - 1) / max_sms_len;
+	count = (octets + max_sms_len - 1) / max_sms_len;
 	dprintf("Will need %d sms-es\n", count);
+	dprintf("SMS is %d octects long but we can only send %d octects in a single SMS after adding %d octects for udh\n", octets, max_sms_len, data->sms->udh.length + 1);
 
 	data->sms->parts = count;
 	data->sms->reference = calloc(count, sizeof(unsigned int));
 
-	start = 0;
-	copied = 0;
-
 	/* Generate reference number */
-	if (!init) {
-		time_t t;
-		time(&t);
-		srand(t);
-		init = 1;
-	}
+	time(&t);
+	srand(t);
 	refnum = (int)(255.0*rand()/(RAND_MAX+1.0));
 
+	/* have a copy of the original data */
+	i = 0;
+	while (data->sms->user_data[i].type != GN_SMS_DATA_None) {
+		memcpy(&ud[i], &data->sms->user_data[i], sizeof(gn_sms_user_data));
+		i++;
+	}
+
 	for (i = 0; i < count; i++) {
 		dprintf("Sending sms #%d (refnum: %d)\n", i+1, refnum);
-		data->sms->udh.udh[isConcat].u.concatenated_short_message.reference_number = refnum;
-		data->sms->udh.udh[isConcat].u.concatenated_short_message.maximum_number = count;
-		data->sms->udh.udh[isConcat].u.concatenated_short_message.current_number = i+1;
+		data->sms->udh.udh[is_concat].u.concatenated_short_message.reference_number = refnum;
+		data->sms->udh.udh[is_concat].u.concatenated_short_message.maximum_number = count;
+		data->sms->udh.udh[is_concat].u.concatenated_short_message.current_number = i+1;
 		switch (data->sms->dcs.u.general.alphabet) {
 		case GN_SMS_DCS_DefaultAlphabet:
 			start += copied;
-			if (ud[0].length - start >= max_sms_len * 8 / 7) {
-				copied = max_sms_len * 8 / 7;
-			} else {
-				copied = (ud[0].length - start) % (max_sms_len * 8 / 7);
-			}
 			memset(&data->sms->user_data[0], 0, sizeof(gn_sms_user_data));
 			data->sms->user_data[0].type = ud[0].type;
-			data->sms->user_data[0].length = copied;
-			memcpy(data->sms->user_data[0].u.text, ud[0].u.text+start, copied);
+			dprintf("%d %d %d\n", start, copied, max_sms_len);
+			copied = char_def_alphabet_string_copy(data->sms->user_data[0].u.text, ud[0].u.text, max_sms_len * 8 / 7, start);
+			dprintf("\tnumber of processed characters: %d\n", copied);
 			break;
 		case GN_SMS_DCS_UCS2:
 			/* We need to copy precisely not to cut character in the middle */
 			start += copied;
 			memset(&data->sms->user_data[0], 0, sizeof(gn_sms_user_data));
 			data->sms->user_data[0].type = ud[0].type;
-			/* We assume UTF8 input */
+			/* FIXME: We assume UTF8 input */
 			size = 1;
 #define C ud[0].u.text[start + j]
 			for (j = 0, k = 0; start + j < ud[0].length && k < max_sms_len / 2; j++) {
@@ -1756,21 +1858,31 @@ static gn_error sms_send_long(gn_data *data, struct gn_statemachine *state)
 					else if (C >= 252 && C < 254)
 						size = 6;
 					else
+						/* FIXME: handle it somehow */
 						dprintf("CHARACTER ENCODING ERROR\n");
-					/* Avoid cutting the character */
-					if (j + size > max_sms_len / 2) {
-						dprintf("DEBUG: break: %d %d %d\n", j, size, max_sms_len / 2);
-						break;
-					}
 					k++;
 				}
-				data->sms->user_data[0].u.text[j] = ud[0].u.text[start + j];
+				/* Avoid cutting a character */
+				if (k < max_sms_len / 2)
+					data->sms->user_data[0].u.text[j] = ud[0].u.text[start + j];
+				else
+					j--;
 			}
 #undef C
 			data->sms->user_data[0].length = copied = j;
 			dprintf("DEBUG: copied: %d\n", copied);
 			break;
 		default:
+			start += copied;
+			if (ud[0].length - start >= max_sms_len) {
+				copied = max_sms_len;
+			} else {
+				copied = (ud[0].length - start) % (max_sms_len);
+			}
+			memset(&data->sms->user_data[0], 0, sizeof(gn_sms_user_data));
+			data->sms->user_data[0].type = ud[0].type;
+			data->sms->user_data[0].length = copied;
+			memcpy(data->sms->user_data[0].u.text, ud[0].u.text+start, copied);
 			switch (ud[0].type) {
 			case GN_SMS_DATA_Bitmap:
 				break;
@@ -1782,7 +1894,7 @@ static gn_error sms_send_long(gn_data *data, struct gn_statemachine *state)
 			break;
 		}
 		dprintf("Text to be sent in this part: %s\n", data->sms->user_data[0].u.text);
-		error = gn_sms_send(data, state);
+		error = sms_send_single(data, state);
 		ERROR();
 	}
 	return GN_ERR_NONE;
@@ -1870,7 +1982,8 @@ char *encode_indication(gn_wap_push *wp, int *data_len)
 	data = malloc(*data_len);
 	
 	if (!data) {
-	    return NULL;
+		free(attr);
+		return NULL;
 	}
 	
 	/* indication tag token */
@@ -2080,7 +2193,7 @@ GNOKII_API char *gn_sms2mbox(gn_sms *sms, char *from)
 	switch (sms->status) {
 	case GN_SMS_Sent:
 	case GN_SMS_Unsent:
-		CONCAT(buf, tmp, size, "To %s@%s %s", sms->remote.number, from, asctime(loctime));
+		CONCAT(buf, tmp, size, "From %s@%s %s", "+0", from, asctime(loctime));
 		break;
 	case GN_SMS_Read:
 	case GN_SMS_Unread:
@@ -2136,9 +2249,7 @@ GNOKII_API char *gn_sms2mbox(gn_sms *sms, char *from)
 
 	return buf;
 error:
-	if (buf)
-		free(buf);
-	if (aux)
-		free(aux);
+	free(buf);
+	free(aux);
 	return NULL;
 }
