@@ -1,7 +1,5 @@
 /*
 
-  $Id: fbus-phonet.c,v 1.47 2007/12/04 19:27:34 hadess Exp $
-
   G N O K I I
 
   A Linux/Unix toolset and driver for the mobile phones.
@@ -24,7 +22,7 @@
 
   Copyright (C) 1999-2000 Hugh Blemings & Pavel Janik ml.
   Copyright (C) 2000-2001 Chris Kemp
-  Copyright (C) 2001-2004 Pawel Kot
+  Copyright (C) 2001-2011 Pawel Kot
   Copyright (C) 2001      Manfred Jonsson, Martin Jancar
   Copyright (C) 2002      Ladis Michl
   Copyright (C) 2002-2003 BORBELY Zoltan
@@ -61,6 +59,24 @@ static gn_error phonet_send_message(unsigned int messagesize, unsigned char mess
 
 #define FBUS_PHONET_BLUETOOTH_INITSEQ 0xd0, 0x00, 0x01
 
+/*--------------------------------------------*/
+
+static int verify_max_message_len(int len, char **message_buffer)
+{
+	static int max_message_len = 0;
+
+	if (len > max_message_len) {
+		dprintf("overrun: %d %d\n", len, max_message_len);
+		*message_buffer = realloc(*message_buffer, len + 1);
+		max_message_len = len + 1;
+	}
+	if (*message_buffer)
+		return max_message_len;
+	else
+		return 0;
+}
+
+
 /*--------------------------------------------*/
 
 static bool phonet_open(struct gn_statemachine *state)
@@ -127,6 +143,7 @@ static void phonet_rx_statemachine(unsigned char rx_byte, struct gn_statemachine
 		    rx_byte == FBUS_PHONET_DKU2_FRAME_ID) {
 			i->state = FBUS_RX_GetDestination;
 		}
+		i->message_corrupted = 0;
 		break;
 
 	case FBUS_RX_GetDestination:
@@ -168,24 +185,35 @@ static void phonet_rx_statemachine(unsigned char rx_byte, struct gn_statemachine
 		i->message_length = i->message_length + rx_byte;
 		i->state = FBUS_RX_GetMessage;
 		i->buffer_count = 0;
+		if (!verify_max_message_len(i->message_length, &(i->message_buffer))) {
+			dprintf("PHONET: Failed to allocate memory for larger buffer\n");
+			dprintf("PHONET: Message buffer overrun - resetting (message length: %d, max: %d)\n", i->message_length, PHONET_FRAME_MAX_LENGTH);
+			i->message_corrupted = 1;
+		}
 		break;
 
 	case FBUS_RX_GetMessage:
+		/*
+		 * If message is corrupted it means that we couldn't allocate full
+		 * space to store it. Therefore let's just print out the message.
+		 */
+		if (i->message_corrupted) {
+			if (i->buffer_count % 16 == 0)
+				dprintf("\n");
+			dprintf("%02x ", rx_byte);
+			i->buffer_count++;
+		}
+
 		i->message_buffer[i->buffer_count] = rx_byte;
 		i->buffer_count++;
 
-		if (i->buffer_count > PHONET_FRAME_MAX_LENGTH) {
-			dprintf("PHONET: Message buffer overun - resetting\n");
-			i->state = FBUS_RX_Sync;
-			break;
-		}
-
 		/* Is that it? */
 		if (i->buffer_count == i->message_length) {
 			sm_incoming_function(i->message_type, i->message_buffer, i->message_length, state);
 			i->state = FBUS_RX_Sync;
 		}
 		break;
+
 	default:
 		i->state = FBUS_RX_Sync;
 		break;
@@ -199,6 +227,69 @@ static void phonet_rx_statemachine(unsigned char rx_byte, struct gn_statemachine
 static gn_error phonet_loop(struct timeval *timeout, struct gn_statemachine *state)
 {
 	gn_error	error = GN_ERR_INTERNALERROR;
+	/*
+	 * IMPORTANT: size of this buffer must be a multiple of packet size for USB compatibility.
+	 * See http://libusb.sourceforge.net/api-1.0/packetoverflow.html
+	 * Packet size can be found in the lsusb output for the proper endpoint. In case of the
+	 * overruns validate this information.
+	    Interface Descriptor:
+	      bLength                 9
+	      bDescriptorType         4
+	      bInterfaceNumber       11
+	      bAlternateSetting       0
+	      bNumEndpoints           0
+	      bInterfaceClass         2 Communications
+	      bInterfaceSubClass    254 
+	      bInterfaceProtocol      0 
+	      iInterface              0 
+	      CDC Header:
+	        bcdCDC               1.10
+	      UNRECOGNIZED CDC:  05 24 ab 05 5c
+	      CDC Union:
+	        bMasterInterface        11
+	        bSlaveInterface         12 
+	    Interface Descriptor:
+	      bLength                 9
+	      bDescriptorType         4
+	      bInterfaceNumber       12
+	      bAlternateSetting       0
+	      bNumEndpoints           0
+	      bInterfaceClass        10 CDC Data
+	      bInterfaceSubClass      0 Unused
+	      bInterfaceProtocol      0 
+	      iInterface              0 
+	    Interface Descriptor:
+	      bLength                 9
+	      bDescriptorType         4
+	      bInterfaceNumber       12
+	      bAlternateSetting       1
+	      bNumEndpoints           2
+	      bInterfaceClass        10 CDC Data
+	      bInterfaceSubClass      0 Unused
+	      bInterfaceProtocol      0 
+	      iInterface              0 
+	      ** UNRECOGNIZED:  04 24 fd 01
+	      Endpoint Descriptor:
+	        bLength                 7
+	        bDescriptorType         5
+	        bEndpointAddress     0x88  EP 8 IN
+	        bmAttributes            2
+	          Transfer Type            Bulk
+	          Synch Type               None
+	          Usage Type               Data
+	        wMaxPacketSize     0x0040  1x 64 bytes		<==
+	        bInterval               0
+	      Endpoint Descriptor:
+	        bLength                 7
+	        bDescriptorType         5
+	        bEndpointAddress     0x08  EP 8 OUT
+	        bmAttributes            2
+	          Transfer Type            Bulk
+	          Synch Type               None
+	          Usage Type               Data
+	        wMaxPacketSize     0x0040  1x 64 bytes		<==
+	        bInterval               0
+	*/
 	unsigned char	buffer[BUFFER_SIZE];
 	int		count, res;
 
@@ -220,7 +311,6 @@ static gn_error phonet_loop(struct timeval *timeout, struct gn_statemachine *sta
 }
 
 /* Main function to send an fbus message */
-
 static gn_error phonet_send_message(unsigned int messagesize, unsigned char messagetype, unsigned char *message, struct gn_statemachine *state)
 {
 
@@ -231,7 +321,9 @@ static gn_error phonet_send_message(unsigned int messagesize, unsigned char mess
 	if (!state)
 		return GN_ERR_FAILED;
 
-	/* FIXME - we should check for the message length ... */
+	if (messagesize > PHONET_TRANSMIT_MAX_LENGTH) {
+		return GN_ERR_MEMORYFULL;
+	}
 
 	/* Now construct the message header. */
 
@@ -290,6 +382,12 @@ static void phonet_reset(struct gn_statemachine *state)
 	FBUSINST(state)->buffer_count = 0;
 }
 
+static void phonet_cleanup(struct gn_statemachine *state)
+{
+	free(FBUSINST(state)->message_buffer);
+	FBUSINST(state)->message_buffer = NULL;
+}
+
 /* Initialise variables and start the link */
 gn_error phonet_initialise(struct gn_statemachine *state)
 {
@@ -302,16 +400,21 @@ gn_error phonet_initialise(struct gn_statemachine *state)
 	state->link.loop = &phonet_loop;
 	state->link.send_message = &phonet_send_message;
 	state->link.reset = &phonet_reset;
+	state->link.cleanup = &phonet_cleanup;
 
 	if ((FBUSINST(state) = calloc(1, sizeof(phonet_incoming_message))) == NULL)
 		return GN_ERR_MEMORYFULL;
 
+	if (!verify_max_message_len(PHONET_FRAME_MAX_LENGTH, &(FBUSINST(state)->message_buffer)))
+		return GN_ERR_MEMORYFULL;
+
 	switch (state->config.connection_type) {
 	case GN_CT_Infrared:
 	case GN_CT_Irda:
 	case GN_CT_DKU2:
 	case GN_CT_DKU2LIBUSB:
 	case GN_CT_Bluetooth:
+	case GN_CT_SOCKETPHONET:
 		if (phonet_open(state) == true)
 			error = GN_ERR_NONE;
 		break;
