@@ -1,7 +1,5 @@
 /*
 
-  $Id: misc.c,v 1.141 2008/09/10 21:03:08 pkot Exp $
-
   G N O K I I
 
   A Linux/Unix toolset and driver for the mobile phones.
@@ -24,7 +22,7 @@
 
   Copyright (C) 1999-2000 Hugh Blemings & Pavel Jank ml.
   Copyright (C) 2000      Jan Derfinak
-  Copyright (C) 2000-2004 Pawel Kot
+  Copyright (C) 2000-2011 Pawel Kot
   Copyright (C) 2001      Jan Kratochvil, Ladis Michl, Chris Kemp
   Copyright (C) 2002-2004 BORBELY Zoltan
   Copyright (C) 2002      Manfred Jonsson, Markus Plail
@@ -50,6 +48,7 @@
 
 #include "misc.h"
 #include "gnokii.h"
+#include "gnokii-internal.h"
 
 GNOKII_API gn_log_target gn_log_debug_mask = GN_LOG_T_NONE;
 GNOKII_API gn_log_target gn_log_rlpdebug_mask = GN_LOG_T_NONE;
@@ -93,9 +92,13 @@ static gn_phone_model models[] = {
 	{"1100",  "RH-18",	PM_OLD_DEFAULT },
 	{"1100",  "RH-38",	PM_OLD_DEFAULT },
 	{"1100b", "RH-36",	PM_OLD_DEFAULT },
+	{"1101",  "RH-75",	PM_OLD_DEFAULT }, /* Yet another 1100 variant, untested */
+	{"1110",  "RH-93",	PM_DEFAULT_S30 },
+	{"1209",  "RH-105",	PM_DEFAULT_S30 },
 	{"1220",  "NKC-1",	PM_OLD_DEFAULT },
 	{"1260",  "NKW-1",	PM_OLD_DEFAULT },
 	{"1261",  "NKW-1C",	PM_OLD_DEFAULT },
+	{"1280",  "RM-647",	PM_DEFAULT_S30 },
 	{"1610",  "NHE-5",	PM_OLD_DEFAULT },
 	{"1610",  "NHE-5NX",	PM_OLD_DEFAULT },
 	{"1611",  "NHE-5",	PM_OLD_DEFAULT },
@@ -125,16 +128,21 @@ static gn_phone_model models[] = {
 	{"2285",  "RH-3",	PM_OLD_DEFAULT },
 	{"2300",  "RM-4",	PM_OLD_DEFAULT },
 	{"2300b", "RM-5",	PM_OLD_DEFAULT },
+	{"2330 classic", "RM-512",	PM_DEFAULT_S40_3RD }, /* Series 40 5th Edition FP 1 Lite */
 	{"2600",  "RH-59",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
 	{"2630",  "RM-298",	PM_DEFAULT },
 	{"2650",  "RH-53",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
+	{"2700 classic",  "RM-561",	PM_DEFAULT_S40_3RD },
+	{"2730 classic",  "RM-578",	PM_DEFAULT_S40_3RD }, /* Series 40 5th Edition FP 1 */
+	{"2760",  "RM-258",	PM_DEFAULT_S40_3RD }, /* Series 40 5th Edition */
 	{"3100",  "RH-19",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
 	{"3100b", "RH-50",	PM_DEFAULT },
 	{"3105",  "RH-48",	PM_OLD_DEFAULT },
 	{"3108",  "RH-6",	PM_OLD_DEFAULT },
+	{"3109c", "RM-274",	PM_DEFAULT_S40_3RD }, /* Nokia 3109 classic */
 	{"3110",  "NHE-8",	PM_OLD_DEFAULT | PM_DATA },
 	{"3110",  "0310" ,	PM_OLD_DEFAULT | PM_DATA }, /* NHE-8 */
-	{"3110c", "RM-237",     PM_DEFAULT | PM_SMSFILE }, /* Nokia 3110 classic */
+	{"3110c", "RM-237",     PM_DEFAULT_S40_3RD }, /* Nokia 3110 classic */
 	{"3120",  "RH-19",	PM_DEFAULT },
 	{"3120b", "RH-50",	PM_DEFAULT },
 	{"3200",  "RH-30",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
@@ -187,6 +195,7 @@ static gn_phone_model models[] = {
 	{"5120",  "NSC-1",	PM_OLD_DEFAULT  },
 	{"5125",  "NSC-1",	PM_OLD_DEFAULT },
 	{"5130",  "NSK-1",	PM_OLD_DEFAULT | PM_NETMONITOR | PM_DATA | PM_AUTHENTICATION },
+	{"5130 XpressMusic",  "RM-495",	PM_DEFAULT_S40_3RD },
 	{"5140",  "NPL-5",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
 	{"5140i", "RM-104",	PM_DEFAULT },
 	{"5160",  "NSW-1",	PM_OLD_DEFAULT | PM_NETMONITOR | PM_DATA | PM_AUTHENTICATION },
@@ -195,26 +204,29 @@ static gn_phone_model models[] = {
 	{"5185",  "NSD-1A",	PM_OLD_DEFAULT },
 	{"5190",  "NSB-1",	PM_OLD_DEFAULT | PM_NETMONITOR | PM_DATA | PM_AUTHENTICATION },
 	{"5210",  "NSM-5",	PM_DEFAULT },
-	{"5300",  "RM-146",	PM_DEFAULT },
+	{"5300 XpressMusic",  "RM-146",	PM_DEFAULT_S40_3RD },
+	{"5310 XpressMusic",  "RM-303",	PM_DEFAULT_S40_3RD },
 	{"5510",  "NPM-5",	PM_OLD_DEFAULT },
 	{"6010",  "NPM-10",	PM_OLD_DEFAULT },
 	{"6010",  "NPM-10X",	PM_OLD_DEFAULT },
 	{"6011",  "RTE-2RH",	PM_OLD_DEFAULT },
-	{"6015i", "RH-55",	PM_DEFAULT | PM_NETMONITOR },
-	{"6020",  "RM-30",	PM_DEFAULT | PM_NETMONITOR },
-	{"6021",  "RM-94",	PM_DEFAULT | PM_NETMONITOR | PM_XGNOKIIBREAKAGE},
+	{"6015i", "RH-55",	PM_DEFAULT },
+	{"6020",  "RM-30",	PM_DEFAULT },
+	{"6021",  "RM-94",	PM_DEFAULT | PM_XGNOKIIBREAKAGE},
 	{"6030",  "RM-74",      PM_DEFAULT },
 	{"6050",  "NME-1",	PM_OLD_DEFAULT },
 	{"6070",  "RM-166",	PM_DEFAULT },
 	{"6080",  "NME-2",	PM_OLD_DEFAULT },
 	{"6081",  "NME-2A",	PM_OLD_DEFAULT },
 	{"6081",  "NME-2E",	PM_OLD_DEFAULT },
+	{"6085",  "RM-260",     PM_DEFAULT },
 	{"6090",  "NME-3",	PM_OLD_DEFAULT },
 	{"6100",  "NPL-2",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
 	{"6108",  "RH-4",	PM_OLD_DEFAULT },
 	{"6110",  "NSE-3",	PM_OLD_DEFAULT | PM_NETMONITOR | PM_DATA | PM_AUTHENTICATION },
 	{"6120",  "NSC-3",	PM_OLD_DEFAULT | PM_NETMONITOR | PM_DATA | PM_AUTHENTICATION },
 	{"6130",  "NSK-3",	PM_OLD_DEFAULT | PM_NETMONITOR | PM_DATA | PM_AUTHENTICATION },
+	{"6133",  "RM-126",	PM_DEFAULT_S40_3RD },
 	{"6136",  "RM-199",	PM_DEFAULT | PM_FULLPBK },
 	{"6138",  "NSK-3",	PM_OLD_DEFAULT },
 	{"6150",  "NSM-1",	PM_OLD_DEFAULT | PM_NETMONITOR | PM_DATA | PM_AUTHENTICATION },
@@ -231,13 +243,14 @@ static gn_phone_model models[] = {
 	{"6210",  "NPE-3",	PM_DEFAULT | PM_NETMONITOR },
 	{"6220",  "RH-20",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
 	{"6225",  "RH-27",	PM_OLD_DEFAULT },
-	{"6230",  "RH-12_28_-",	PM_DEFAULT | PM_NETMONITOR },
-	{"6230",  "RH12_28_-",	PM_DEFAULT | PM_NETMONITOR },
-	{"6230i", "RM72_73_-",	PM_DEFAULT | PM_NETMONITOR },
-	{"6233",  "RM123_145_-", PM_DEFAULT | PM_NETMONITOR },
+	{"6230",  "RH-12_28_-",	PM_DEFAULT },
+	{"6230",  "RH12_28_-",	PM_DEFAULT },
+	{"6230i", "RM72_73_-",	PM_DEFAULT },
+	{"6233",  "RM123_145_-", PM_DEFAULT },
 	{"6250",  "NHM-3",	PM_DEFAULT | PM_NETMONITOR },
-	{"6280",  "RH-78",	PM_DEFAULT | PM_NETMONITOR },
-	{"6300",  "RM-217",     PM_DEFAULT | PM_FULLPBK },
+	{"6280",  "RH-78",	PM_DEFAULT },
+	{"6300",  "RM-217",     PM_DEFAULT_S40_3RD | PM_FULLPBK },
+	{"6303",  "RM-443",     PM_DEFAULT_S40_3RD | PM_FULLPBK }, /* Nokia 6303 classic */
 	{"6310",  "NPE-4",	PM_DEFAULT },
 	{"6310i", "NPL-1",	PM_DEFAULT },
 	{"6340",  "NPM-2",	PM_OLD_DEFAULT },
@@ -245,7 +258,8 @@ static gn_phone_model models[] = {
 	{"6360",  "NPW-2",	PM_DEFAULT },
 	{"6370",  "NHP-2FX",	PM_OLD_DEFAULT },
 	{"6385",  "NHP-2AX",	PM_OLD_DEFAULT },
-	{"6500",  "NHM-7",	PM_DEFAULT | PM_NETMONITOR },
+	{"6500",  "NHM-7",	PM_DEFAULT },
+	{"6500c", "RM-265",	PM_DEFAULT_S40_3RD | PM_FULLPBK }, /* Series 40 5th Edition */
 	{"6510",  "NPM-9",	PM_DEFAULT },
 	{"6560",  "RH-25",	PM_OLD_DEFAULT },
 	{"6585",  "RH-34",	PM_OLD_DEFAULT },
@@ -255,7 +269,8 @@ static gn_phone_model models[] = {
 	{"6610",  "NHL-4U",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
 	{"6610i", "RM-37",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
 	{"6650",  "NHM-1",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
-	{"6680",  "RM-36",	PM_DEFAULT | PM_NETMONITOR },
+	{"6680",  "RM-36",	PM_DEFAULT },
+	{"6700 classic",  "RM-470",	PM_DEFAULT_S40_3RD },  /* Series 40 6th Edition */
 	{"6800",  "NHL-6",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
 	{"6800",  "NSB-9",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
 	{"6810",  "RM-2",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
@@ -292,7 +307,7 @@ static gn_phone_model models[] = {
 	{"8270",  "NSD-5FX",	PM_OLD_DEFAULT },
 	{"8280",  "RH-10",	PM_OLD_DEFAULT },
 	{"8290",  "NSB-7",	PM_OLD_DEFAULT | PM_NETMONITOR | PM_DATA },
-	{"8310",  "NHM-7",	PM_DEFAULT | PM_NETMONITOR },
+	{"8310",  "NHM-7",	PM_DEFAULT  },
 	{"8390",  "NSB-8",	PM_OLD_DEFAULT },
 	{"8810",  "NSE-6",	PM_OLD_DEFAULT | PM_DATA },
 	{"8850",  "NSM-2",	PM_OLD_DEFAULT | PM_NETMONITOR | PM_DATA },
@@ -312,6 +327,8 @@ static gn_phone_model models[] = {
 	{"N70",   "RM-84",      PM_DEFAULT },
 	{"N73",   "RM-133",	PM_DEFAULT },
 	{"RPM-1", "RPM-1",	PM_OLD_DEFAULT | PM_DATA },
+	{"C1-01", "RM-607",	PM_DEFAULT_S40_3RD },  /* Series 40 6th Edition Lite */
+	{"C2-03", "RM-70",	PM_DEFAULT_S40_3RD },
 	{"Card Phone 1.0", "RPE-1",	PM_OLD_DEFAULT | PM_DATA },
 	{"Card Phone 2.0", "RPM-1",	PM_OLD_DEFAULT | PM_DATA },
 	{"C110 Wireless LAN Card", "DTN-10",	PM_OLD_DEFAULT },
@@ -320,7 +337,7 @@ static gn_phone_model models[] = {
 	{"N-Gage", "NEM-4",	PM_OLD_DEFAULT },
 	{"RinGo", "NHX-7",	PM_OLD_DEFAULT },
 	{"sx1",  "SX1",		PM_DEFAULT },
-	{"SIM/USIM", "pcsc",	0 },
+	{"SIM/USIM", "APDU",	0 },
 	{NULL,    NULL, 0 } /* keep last one as NULL */
 };
 
@@ -352,7 +369,7 @@ GNOKII_API gn_phone_model *gn_phone_model_get(const char *product_name)
 
 GNOKII_API const char *gn_model_get(const char *product_name)
 {
-	return (gn_phone_model_get(product_name)->model);
+	return gn_cfg_get_phone_model(gn_cfg_info, product_name)->model;
 }
 
 static void log_printf(gn_log_target mask, const char *fmt, va_list ap)
@@ -446,7 +463,7 @@ GNOKII_API char *gn_device_lock(const char* port)
 	int fd, len;
 
 	if (!port) {
-		fprintf(stderr, _("Cannot lock NULL device.\n"));
+		fprintf(stderr, _("Cannot lock NULL device. Set port config parameter correctly.\n"));
 		return NULL;
 	}
 
@@ -464,7 +481,8 @@ GNOKII_API char *gn_device_lock(const char* port)
 		fprintf(stderr, _("Out of memory error while locking device.\n"));
 		return NULL;
 	}
-	/* I think we don't need to use strncpy, as we should have enough
+	/*
+	 * I think we don't need to use strncpy, as we should have enough
 	 * buffer due to strlen results, but it's safer to do so...
 	 */
 	strncpy(lock_file, lock_path, len);
@@ -489,15 +507,15 @@ GNOKII_API char *gn_device_lock(const char* port)
 				sscanf(buf, "%d", &pid);
 			}
 			if (pid > 0 && kill((pid_t)pid, 0) < 0 && errno == ESRCH) {
-				fprintf(stderr, _("Lockfile %s is stale. Overriding it..\n"), lock_file);
+				fprintf(stderr, _("Lockfile %s is stale. Overriding it...\n"), lock_file);
 				sleep(1);
 				if (unlink(lock_file) == -1) {
-					fprintf(stderr, _("Overriding failed, please check the permissions.\n"));
+					fprintf(stderr, _("Overriding file %s failed, please check the permissions.\n"), lock_file);
 					fprintf(stderr, _("Cannot lock device.\n"));
 					goto failed;
 				}
 			} else {
-				fprintf(stderr, _("Device already locked.\n"));
+				fprintf(stderr, _("Device already locked with %s.\n"), lock_file);
 				goto failed;
 			}
 		}
@@ -521,7 +539,7 @@ GNOKII_API char *gn_device_lock(const char* port)
 		else if (errno == EACCES)
 			fprintf(stderr, _("Please check permission on lock directory.\n"));
 		else if (errno == ENOENT)
-			fprintf(stderr, _("Cannot create lockfile %s. Please check for existence of the path."), lock_file);
+			fprintf(stderr, _("Cannot create lockfile %s. Please check for existence of the path.\n"), lock_file);
 		goto failed;
 	}
 	snprintf(buffer, sizeof(buffer), "%10ld gnokii\n", (long)getpid());
@@ -542,18 +560,20 @@ failed:
 }
 
 /* Removes lock and frees memory */
-GNOKII_API bool gn_device_unlock(char *lock_file)
+GNOKII_API int gn_device_unlock(char *lock_file)
 {
 #ifndef WIN32
 	int err;
 
-	if (!lock_file) {
-		fprintf(stderr, _("Cannot unlock device.\n"));
-		return false;
+	if (lock_file) {
+		err = unlink(lock_file);
+		free(lock_file);
+		if (err) {
+			fprintf(stderr, _("Cannot unlock device: %s\n"), strerror(errno));
+			return false;
+		}
 	}
-	err = unlink(lock_file);
-	free(lock_file);
-	return (err + 1);
+	return true;
 #else
 	return true;
 #endif /* WIN32 */
@@ -619,6 +639,35 @@ void gnokii_strfreev(char **str_array)
 	free(str_array);
 }
 
+/**
+ * gnokii_strcmpsep:
+ * @s1: a string, NUL terminated, may NOT contain @sep
+ * @s2: a string, NUL terminated, may contain @sep
+ * @sep: comparison stops if this char is found in @s2
+ *
+ * Returns: < 0 if @s1 < @s2
+ *            0 if @s1 == @s2
+ *          > 0 if @s1 > @s2
+ *
+ * Compares two strings up to a NUL terminator or a separator char.
+ * Leading and trailing white space in @s2 is ignored.
+ */
+int gnokii_strcmpsep(const char *s1, const char *s2, char sep)
+{
+	while (isspace(*s2))
+		s2++;
+	while (*s1 && *s1 == *s2) {
+		s1++;
+		s2++;
+	}
+	while (isspace(*s2))
+		s2++;
+	if (!*s1 && *s2 == sep)
+		return 0;
+
+	return *s1 - *s2;
+}
+
 /*
  * check if the timestamp in dt has valid date and time
  */
