@@ -128,6 +128,12 @@ gn_error phonebook_decode(unsigned char *blockstart, int length, gn_data *data,
 				blockstart[6], blockstart[7], blockstart[8], blockstart[9], blockstart[10], blockstart[11]);
 			memcpy(subentry->data.fileid, blockstart+6, 6);
 			break;
+		case GN_PHONEBOOK_ENTRY_Video:
+			dprintf("Video\n");
+			dprintf("   Video id: %02x %02x %02x %02x %02x %02x\n",
+				blockstart[6], blockstart[7], blockstart[8], blockstart[9], blockstart[10], blockstart[11]);
+			memcpy(subentry->data.fileid, blockstart+6, 6);
+			break;
 		case GN_PHONEBOOK_ENTRY_PostalAddress:
 			dprintf("Postal Address %d parts\n", blockstart[7]);
 			blocks += blockstart[7];
@@ -290,6 +296,18 @@ gn_error phonebook_decode(unsigned char *blockstart, int length, gn_data *data,
 				data->bitmap->number = blockstart[5] - 1;
 			dprintf("   Group: %d\n", blockstart[5] - 1);
 			break;
+		case GN_PHONEBOOK_ENTRY_ExtGroup: /* Extended caller group number: in Series40 3rd Ed and later groups are defined by users */
+			if (!subentry) {
+				dprintf("ERROR!!!");
+				break;
+			}
+			subentry->entry_type  = blockstart[0];
+			subentry->number_type = 0;
+			subentry->id          = blockstart[4];
+			subentry->data.id     = blockstart[7];
+			subblock_count++;
+			data->phonebook_entry->subentries_count++;
+			break;
 		case GN_PHONEBOOK_ENTRY_RingtoneAdv:   /* Newer ringtones */
 			switch(blockstart[15]) {
 			case 0x01:
@@ -315,6 +333,12 @@ gn_error phonebook_decode(unsigned char *blockstart, int length, gn_data *data,
 			break;
 		default:
 			dprintf("Unknown phonebook block %02x\n", blockstart[0]);
+			{
+				int j;
+				for (j = 0; j < blockstart[3]; j++)
+					dprintf("%02x ", blockstart[j]);
+				dprintf("\n");
+			}
 			break;
 		}
 		blockstart += blockstart[3];
