@@ -1,6 +1,6 @@
 /*
 
-  $Id: gnapplet.c,v 1.33 2006/10/03 21:26:37 pkot Exp $
+  $Id$
 
   G N O K I I
 
@@ -98,8 +98,6 @@ static gn_error gnapplet_sms_message_write(gn_data *data, struct gn_statemachine
 static gn_error gnapplet_sms_message_send(gn_data *data, struct gn_statemachine *state);
 static gn_error gnapplet_sms_message_delete_nv(gn_data *data, struct gn_statemachine *state);
 static gn_error gnapplet_sms_message_delete(gn_data *data, struct gn_statemachine *state);
-static gn_error gnapplet_sms_message_move_nv(gn_data *data, struct gn_statemachine *state);
-static gn_error gnapplet_sms_message_move(gn_data *data, struct gn_statemachine *state);
 static gn_error gnapplet_sms_center_read(gn_data *data, struct gn_statemachine *state);
 static gn_error gnapplet_sms_center_write(gn_data *data, struct gn_statemachine *state);
 static gn_error gnapplet_calendar_note_read(gn_data *data, struct gn_statemachine *state);
@@ -115,6 +113,7 @@ static gn_error gnapplet_clock_alarm_read(gn_data *data, struct gn_statemachine
 static gn_error gnapplet_clock_alarm_write(gn_data *data, struct gn_statemachine *state);
 static gn_error gnapplet_profile_read(gn_data *data, struct gn_statemachine *state);
 static gn_error gnapplet_profile_active_read(gn_data *data, struct gn_statemachine *state);
+static gn_error gnapplet_profile_active_write(gn_data *data, struct gn_statemachine *state);
 
 static gn_error gnapplet_incoming_info(int messagetype, unsigned char *message, int length, gn_data *data, struct gn_statemachine *state);
 static gn_error gnapplet_incoming_phonebook(int messagetype, unsigned char *message, int length, gn_data *data, struct gn_statemachine *state);
@@ -465,12 +464,6 @@ static gn_error gnapplet_functions(gn_operation op, gn_data *data, struct gn_sta
 		return gnapplet_sms_message_delete_nv(data, state);
 	case GN_OP_DeleteSMS:
 		return gnapplet_sms_message_delete(data, state);
-	/*
-	case GN_OP_MoveSMSnoValidate:
-		return gnapplet_sms_message_move_nv(data, state);
-	case GN_OP_MoveSMS:
-		return gnapplet_sms_message_move(data, state);
-	*/
 	case GN_OP_GetSMSCenter:
 		return gnapplet_sms_center_read(data, state);
 	case GN_OP_SetSMSCenter:
@@ -499,6 +492,8 @@ static gn_error gnapplet_functions(gn_operation op, gn_data *data, struct gn_sta
 		return gnapplet_profile_read(data, state);
 	case GN_OP_GetActiveProfile:
 		return gnapplet_profile_active_read(data, state);
+	case GN_OP_SetActiveProfile:
+		return gnapplet_profile_active_write(data, state);
 	default:
 		dprintf("gnapplet unimplemented operation: %d\n", op);
 		return GN_ERR_NOTIMPLEMENTED;
@@ -870,7 +865,7 @@ static gn_error gnapplet_incoming_power(int messagetype, unsigned char *message,
 		percent = pkt_get_uint8(&pkt);
 		source = pkt_get_uint8(&pkt);
 		if (error != GN_ERR_NONE) return error;
-		if (data->battery_unit) *data->battery_unit = GN_RF_Percentage;
+		if (data->battery_unit) *data->battery_unit = GN_BU_Percentage;
 		if (data->battery_level) *data->battery_level = percent;
 		if (data->power_source) *data->power_source = source;
 		break;
@@ -1084,7 +1079,7 @@ static gn_error gnapplet_sms_message_delete(gn_data *data, struct gn_statemachin
 	return gnapplet_sms_message_delete_nv(data, state);
 }
 
-
+#if 0
 static gn_error gnapplet_sms_message_move_nv(gn_data *data, struct gn_statemachine *state)
 {
 	REQUEST_DEF;
@@ -1111,7 +1106,7 @@ static gn_error gnapplet_sms_message_move(gn_data *data, struct gn_statemachine
 
 	return gnapplet_sms_message_move_nv(data, state);
 }
-
+#endif
 
 static gn_error gnapplet_sms_center_read(gn_data *data, struct gn_statemachine *state)
 {
@@ -1571,6 +1566,19 @@ static gn_error gnapplet_profile_active_read(gn_data *data, struct gn_statemachi
 }
 
 
+static gn_error gnapplet_profile_active_write(gn_data *data, struct gn_statemachine *state)
+{
+	REQUEST_DEF;
+
+	if (!data->profile) return GN_ERR_INTERNALERROR;
+
+	pkt_put_uint16(&pkt, GNAPPLET_MSG_PROFILE_SET_ACTIVE_REQ);
+	pkt_put_uint16(&pkt, data->profile->number);
+
+	SEND_MESSAGE_BLOCK(GNAPPLET_MSG_PROFILE);
+}
+
+
 static gn_error gnapplet_incoming_profile(int messagetype, unsigned char *message, int length, gn_data *data, struct gn_statemachine *state)
 {
 	gn_profile *profile;
@@ -1602,6 +1610,11 @@ static gn_error gnapplet_incoming_profile(int messagetype, unsigned char *messag
 		profile->number = pkt_get_uint16(&pkt);
 		break;
 
+	case GNAPPLET_MSG_PROFILE_SET_ACTIVE_RESP:
+		if (!(profile = data->profile)) return GN_ERR_INTERNALERROR;
+		if (error != GN_ERR_NONE) return error;
+		break;
+
 	default:
 		return GN_ERR_UNHANDLEDFRAME;
 	}
