@@ -199,7 +199,6 @@ static gn_error get_gn_error(PCSC_IOSTRUCT *ios, LONG ret)
 	case SCARD_W_UNPOWERED_CARD:
 	case SCARD_W_RESET_CARD:
 	case SCARD_W_REMOVED_CARD:
-	case SCARD_W_INSERTED_CARD:
 	 	return GN_ERR_SIMPROBLEM;
 	case SCARD_E_NO_MEMORY:
 		return GN_ERR_MEMORYFULL;
@@ -427,13 +426,33 @@ static gn_error get_phase(const char **phase)
 		case 1: *phase = "1"; break;
 		case 2: *phase = "2"; break;
 		case 3: *phase = "2+"; break;
-		default: dprintf("Unknown Phase Identification 0x%02d\n", IoStruct.pbRecvBuffer[0]);
+		default: dprintf("Unknown Phase Identification 0x%02x\n", IoStruct.pbRecvBuffer[0]);
 		}
 	}
 
 	return error;
 }
 
+/* read SPN and convert it into an ASCII string in the provided buffer (at least GN_PCSC_SPN_MAX_LENGTH bytes long)
+according to subclause 10.3.11 and Annex B
+*/
+static gn_error get_spn(char *result)
+{
+	LONG ret;
+	BYTE *buf;
+	gn_error error;
+
+	ret = pcsc_read_file(&IoStruct, GN_PCSC_FILE_DF_GSM, GN_PCSC_FILE_EF_SPN);
+	error = get_gn_error(&IoStruct, ret);
+	if (error != GN_ERR_NONE) return error;
+
+	/* Skip first byte (it's a bit field) */
+	buf = IoStruct.pbRecvBuffer + 1;
+	alpha_tag_decode(result, buf, GN_PCSC_SPN_MAX_LENGTH);
+
+	return GN_ERR_NONE;
+}
+
 /* functions for libgnokii stuff */
 
 static gn_error functions(gn_operation op, gn_data *data, struct gn_statemachine *state)
@@ -533,6 +552,7 @@ static gn_error Identify(gn_data *data, struct gn_statemachine *state)
 {
 	gn_error error;
 	char imsi[GN_PCSC_IMSI_MAX_LENGTH], iccid[GN_PCSC_ICCID_MAX_LENGTH];
+	char spn[GN_PCSC_SPN_MAX_LENGTH] = "";
 	const char *phase = "??";
 
 	/* read ICCID */
@@ -545,7 +565,7 @@ static gn_error Identify(gn_data *data, struct gn_statemachine *state)
 	/* read IMSI */
 	error = get_imsi(imsi);
 	if (error == GN_ERR_NONE) {
-		char mcc[4], mnc[4], net_code[8];
+		char mcc[4], mnc[4], net_code[8], *network_name;
 		int mnc_len = 2;
 
 		/* extract MCC and MNC */
@@ -556,10 +576,16 @@ static gn_error Identify(gn_data *data, struct gn_statemachine *state)
 		dprintf("IMSI = %s Home PLMN = MCC %s MNC %s\n", imsi, mcc, mnc);
 		snprintf(net_code, sizeof(net_code), "%s %s", mcc, mnc);
 		/* get operator and country names as strings from libgnokii */
-		snprintf(data->manufacturer, GN_MANUFACTURER_MAX_LENGTH, "%s (%s)", gn_network_name_get(net_code), gn_country_name_get(mcc));
+		network_name = gn_network_name_get(net_code);
+		/* read Service Provider Name */
+		(void) get_spn(spn);
+		if (*spn && strcmp(spn, network_name))
+			snprintf(data->manufacturer, GN_MANUFACTURER_MAX_LENGTH, "%s, %s (%s)", spn, network_name, gn_country_name_get(mcc));
+		else
+			snprintf(data->manufacturer, GN_MANUFACTURER_MAX_LENGTH, "%s (%s)", network_name, gn_country_name_get(mcc));
 	}
 
-	snprintf(data->model, GN_MODEL_MAX_LENGTH, "%s", "pcsc");
+	snprintf(data->model, GN_MODEL_MAX_LENGTH, "%s", "APDU");
 
 	/* read SIM "Phase Identification" */
 	/* ignore errors, set default value above */
@@ -1222,7 +1248,8 @@ static LONG pcsc_cmd_read_record(PCSC_IOSTRUCT *ios, BYTE record, BYTE length)
 
 static LONG pcsc_file_get_contents(PCSC_IOSTRUCT *ios, LONG file_id)
 {
-/* copy file contents in a newly allocated buffer which must be freed by the caller
+/* copy file contents in a newly allocated buffer ios->pbRecvBuffer which must
+be freed by the caller also in case of error return.
 Note that this function overwrites ios->pbRecvBuffer and ios->dwRecvLength
 */
 	LONG ret;
@@ -1231,7 +1258,7 @@ Note that this function overwrites ios->pbRecvBuffer and ios->dwRecvLength
 
 	/* allocate a buffer for the "stat" command */
 	ios->dwRecvLength = MAX_BUFFER_SIZE;
-	ios->pbRecvBuffer = malloc(ios->dwRecvLength);  /* FIXME this memory is leaked after the following errors */
+	ios->pbRecvBuffer = malloc(ios->dwRecvLength);
 	if (!ios->pbRecvBuffer) return SCARD_E_NO_MEMORY;
 
 	ret = pcsc_stat_file(ios, file_id);
