@@ -58,7 +58,7 @@ static char *ical_fgets(char *s, size_t size, void *d)
 
 /* This string is used so often that we keep it as long as the program is
  * running. Due to VERSION being defined in both libical and gnokii we need
- * to us a static string instead of a pre-compiler macro.
+ * to use a static string instead of a pre-processor macro.
  */
 static char vcal_prodid[64] = {0};
 static inline const char *get_prodid()
@@ -107,16 +107,13 @@ static void ical_append_printf(ical_string *str, const char *fmt, ...)
 /*
 	ICALENDAR Reading functions
  */
-GNOKII_API char * gn_calnote2icalstr (gn_calnote *calnote)
+GNOKII_API char *gn_calnote2icalstr(gn_calnote *calnote)
 {
 	ical_string str;
 
 #ifdef HAVE_LIBICAL
-#  define MAX_PROP_INDEX 5
-	icalcomponent *pIcal = NULL;
-	struct icaltimetype stime = {0}, etime = {0}/*, atime = {0}*/;
-	icalproperty *properties[MAX_PROP_INDEX+1] = {0}; /* order and number of properties vary */
-	int iprop = 0;
+	icalcomponent *pIcal = NULL, *vevent;
+	struct icaltimetype stime, etime;
 	char compuid[64];
 
 	memset(&str, 0, sizeof (str));
@@ -124,7 +121,7 @@ GNOKII_API char * gn_calnote2icalstr (gn_calnote *calnote)
 	/* In at least some Nokia phones it is possible to skip the year of
 	   birth, in this case year == 0xffff, so we set it to the arbitrary
 	   value of 1800 */
-
+	stime = icaltime_null_time();
 	stime.year = (calnote->time.year == 0xffff ? 1800 : calnote->time.year);
 	stime.month = calnote->time.month;
 	stime.day = calnote->time.day;
@@ -134,7 +131,19 @@ GNOKII_API char * gn_calnote2icalstr (gn_calnote *calnote)
 
 	stime.is_daylight = 1;
 
+	snprintf(compuid, sizeof(compuid), "guid.gnokii.org_%d_%d", calnote->location, rand());
+
+	vevent = icalcomponent_vanew(ICAL_VEVENT_COMPONENT,
+				     icalproperty_new_uid(compuid),
+				     icalproperty_new_categories("GNOKII"),
+				     0);
+	if (!vevent) {
+		dprintf("ERROR in icalcomponent_vanew() creating VEVENT\n");
+		return NULL;
+	}
+
 	if (calnote->end_time.year) {
+		etime = icaltime_null_time();
 		etime.year = (calnote->end_time.year == 0xffff ? 1800 : calnote->end_time.year);
 		etime.month = calnote->end_time.month;
 		etime.day = calnote->end_time.day;
@@ -143,76 +152,112 @@ GNOKII_API char * gn_calnote2icalstr (gn_calnote *calnote)
 		etime.second = calnote->end_time.second;
 		etime.is_daylight = 1;
 
-		properties[iprop++] = icalproperty_new_dtend(etime);
+		icalcomponent_add_property(vevent, icalproperty_new_dtend(etime));
 	}
 
-	/* FIXME: how to set alarm?
-	if (calnote->alarm.enabled) {
-		atime.year = (calnote->alarm.timestamp.year == 0xffff ? 1800 : calnote->alarm.timestamp.year);
-		atime.month = calnote->alarm.timestamp.month;
-		atime.day = calnote->alarm.timestamp.day;
-		atime.hour = calnote->alarm.timestamp.hour;
-		atime.minute = calnote->alarm.timestamp.minute;
-		atime.second = calnote->alarm.timestamp.second;
-		atime.is_daylight = 1;
-
-		properties[iprop++] = icalproperty_new_FIXME(atime);
-	}
-	*/
-
 	/* TODO: should the strings be configurable? */
 	switch(calnote->type) {
 	case GN_CALNOTE_MEMO:
-		properties[iprop++] = icalproperty_new_categories("MISCELLANEOUS");
-		properties[iprop++] = icalproperty_new_summary(calnote->text);
+		icalcomponent_add_property(vevent, icalproperty_new_categories("MISCELLANEOUS"));
+		icalcomponent_add_property(vevent, icalproperty_new_summary(calnote->text));
 		break;
 	case GN_CALNOTE_REMINDER:
-		properties[iprop++] = icalproperty_new_categories("REMINDER");
-		properties[iprop++] = icalproperty_new_summary(calnote->text);
+		icalcomponent_add_property(vevent, icalproperty_new_categories("REMINDER"));
+		icalcomponent_add_property(vevent, icalproperty_new_summary(calnote->text));
 		break;
 	case GN_CALNOTE_CALL:
-		properties[iprop++] = icalproperty_new_categories("PHONE CALL");
-		properties[iprop++] = icalproperty_new_summary(calnote->phone_number);
-		properties[iprop++] = icalproperty_new_description(calnote->text);
+		icalcomponent_add_property(vevent, icalproperty_new_categories("PHONE CALL"));
+		icalcomponent_add_property(vevent, icalproperty_new_summary(calnote->phone_number));
+		icalcomponent_add_property(vevent, icalproperty_new_description(calnote->text));
 		break;
 	case GN_CALNOTE_MEETING:
-		properties[iprop++] = icalproperty_new_categories("MEETING");
-		properties[iprop++] = icalproperty_new_summary(calnote->text);
+		icalcomponent_add_property(vevent, icalproperty_new_categories("MEETING"));
+		icalcomponent_add_property(vevent, icalproperty_new_summary(calnote->text));
 		if (calnote->mlocation[0])
-			properties[iprop++] = icalproperty_new_location(calnote->mlocation);
+			icalcomponent_add_property(vevent, icalproperty_new_location(calnote->mlocation));
 		break;
 	case GN_CALNOTE_BIRTHDAY:
-		properties[iprop++] = icalproperty_new_categories("ANNIVERSARY");
-		properties[iprop++] = icalproperty_new_summary(calnote->text);
-		do {
-			char rrule[64];
-			snprintf(rrule, sizeof(rrule), "FREQ=YEARLY;INTERVAL=1;BYMONTH=%d", stime.month);
-			properties[iprop++] = icalproperty_new_rrule(icalrecurrencetype_from_string(rrule));
-		} while (0);
+		icalcomponent_add_property(vevent, icalproperty_new_categories("ANNIVERSARY"));
+		icalcomponent_add_property(vevent, icalproperty_new_summary(calnote->text));
 		stime.is_date = 1;
 		calnote->recurrence = GN_CALNOTE_YEARLY;
 		break;
 	default:
-		properties[iprop++] = icalproperty_new_categories("UNKNOWN");
-		properties[iprop++] = icalproperty_new_summary(calnote->text);
+		icalcomponent_add_property(vevent, icalproperty_new_categories("UNKNOWN"));
+		icalcomponent_add_property(vevent, icalproperty_new_summary(calnote->text));
 		break;
 	}
+	icalcomponent_add_property(vevent, icalproperty_new_dtstart(stime));
 
-	snprintf(compuid, sizeof(compuid), "guid.gnokii.org_%d_%d", calnote->location, rand());
+	if (calnote->recurrence) {
+		char rrule[64];
+		char *freq;
+		int interval = 1;
+
+		switch (calnote->recurrence) {
+		case GN_CALNOTE_NEVER:
+			goto norecurrence;
+		case GN_CALNOTE_DAILY:
+			freq = "DAILY";
+			break;
+		case GN_CALNOTE_WEEKLY:
+			freq = "WEEKLY";
+			break;
+		case GN_CALNOTE_2WEEKLY:
+			freq = "WEEKLY";
+			interval = 2;
+			break;
+		case GN_CALNOTE_MONTHLY:
+			freq = "MONTHLY";
+			break;
+		case GN_CALNOTE_YEARLY:
+			freq = "YEARLY";
+			break;
+		default:
+			freq = "HOURLY";
+			interval = calnote->recurrence;
+			break;
+		}
+		if (calnote->type == GN_CALNOTE_BIRTHDAY)
+			snprintf(rrule, sizeof(rrule), "FREQ=YEARLY;INTERVAL=1;BYMONTH=%d", stime.month);
+		else if (calnote->occurrences == 0)
+			snprintf(rrule, sizeof(rrule), "FREQ=%s;INTERVAL=%d", freq, interval);
+		else
+			snprintf(rrule, sizeof(rrule), "FREQ=%s;COUNT=%d;INTERVAL=%d", freq, calnote->occurrences, interval);
+		icalcomponent_add_property(vevent, icalproperty_new_rrule(icalrecurrencetype_from_string(rrule)));
+	}
+norecurrence:
+
+	if (calnote->alarm.enabled) {
+		icalcomponent *valarm;
+		struct icaltriggertype trig;
+
+		trig.time.year = (calnote->alarm.timestamp.year == 0xffff ? 1800 : calnote->alarm.timestamp.year);
+		trig.time.month = calnote->alarm.timestamp.month;
+		trig.time.day = calnote->alarm.timestamp.day;
+		trig.time.hour = calnote->alarm.timestamp.hour;
+		trig.time.minute = calnote->alarm.timestamp.minute;
+		trig.time.second = calnote->alarm.timestamp.second;
+		trig.time.is_daylight = 1;
+
+		valarm = icalcomponent_new_valarm();
+
+		icalcomponent_add_property(valarm, icalproperty_new_trigger(trig));
+		/* With ICAL_ACTION_DISPLAY a DESCRIPTION property is mandatory */
+		if (calnote->alarm.tone) {
+			icalcomponent_add_property(valarm, icalproperty_new_action(ICAL_ACTION_AUDIO));
+		} else {
+			icalcomponent_add_property(valarm, icalproperty_new_action(ICAL_ACTION_DISPLAY));
+			icalcomponent_add_property(valarm, icalproperty_new_description(calnote->text));
+		}
+
+		icalcomponent_add_component(vevent, valarm);
+	}
 
 	pIcal = icalcomponent_vanew(ICAL_VCALENDAR_COMPONENT,
 				    icalproperty_new_version("2.0"),
 				    icalproperty_new_prodid(get_prodid()),
-				    icalcomponent_vanew(ICAL_VEVENT_COMPONENT,
-							icalproperty_new_dtstart(stime),
-							icalproperty_new_uid(compuid),
-							icalproperty_new_categories("GNOKII"),
-							properties[0],
-							properties[1],
-							properties[2],
-							properties[3],
-							properties[4],
-							0),
+				    vevent,
 				    0);
 
 	if (pIcal) {
@@ -229,13 +274,12 @@ GNOKII_API char * gn_calnote2icalstr (gn_calnote *calnote)
 		}
 		ical_append_printf(&str, "%s\n", icalstr);
 		dprintf("%s\n", icalstr);
-		if (icalstrbuf)
-			free(icalstrbuf);
+		free(icalstrbuf);
 
 		icalcomponent_free(pIcal);
 		pIcal = NULL;
 	} else {
-		dprintf("ERROR in icalcomponent_vanew()\n");
+		dprintf("ERROR in icalcomponent_vanew() creating VCALENDAR\n");
 	}
 	return str.str;
 #else
@@ -335,7 +379,7 @@ GNOKII_API int gn_calnote2ical(FILE *f, gn_calnote *calnote)
 	if (vcal == NULL)
 		return -1;
 	retval = fputs(vcal, f);
-	free (vcal);
+	free(vcal);
 	return retval;
 }
 
@@ -374,7 +418,10 @@ static int gn_ical2calnote_real(icalcomponent *comp, gn_calnote *calnote, int id
 	} else {
 		const char *str;
 		icalcomponent *alarm = {0};
+		icalproperty *rrule;
+		struct icalrecurrencetype recur;
 		retval = GN_ERR_NONE;
+
 		calnote->phone_number[0] = 0;
 
 		/* type */
@@ -427,7 +474,7 @@ static int gn_ical2calnote_real(icalcomponent *comp, gn_calnote *calnote, int id
 		}
 
 		/* alarm */
-		alarm = icalcomponent_get_first_component(comp, ICAL_VALARM_COMPONENT);
+		alarm = icalcomponent_get_first_component(compresult, ICAL_VALARM_COMPONENT);
 		if (alarm) {
 			icalproperty *trigger = NULL;
 
@@ -435,10 +482,10 @@ static int gn_ical2calnote_real(icalcomponent *comp, gn_calnote *calnote, int id
 			if (trigger) {
 				struct icaltriggertype trigger_value;
 				struct icaltimetype alarm_start;
+				icalproperty *property;
 
 				trigger_value = icalvalue_get_trigger(icalproperty_get_value(trigger));
-				if (icaltriggertype_is_null_trigger(trigger_value) ||
-						icaltriggertype_is_bad_trigger(trigger_value)) {
+				if (icaltriggertype_is_bad_trigger(trigger_value)) {
 					calnote->alarm.enabled = 0;
 				} else {
 					if (icaltime_is_null_time(trigger_value.time)) {
@@ -458,19 +505,71 @@ static int gn_ical2calnote_real(icalcomponent *comp, gn_calnote *calnote, int id
 					calnote->alarm.timestamp.minute = alarm_start.minute;
 					calnote->alarm.timestamp.second = alarm_start.second;
 					/* TODO: time zone? */
+
+					property = icalcomponent_get_first_property(alarm, ICAL_ACTION_PROPERTY);
+					calnote->alarm.tone = icalproperty_get_action(property) == ICAL_ACTION_AUDIO ? 1 : 0;
 				}
 			}
 		}
 
+		/* recurrence */
+		rrule = icalcomponent_get_first_property(compresult, ICAL_RRULE_PROPERTY);
+		if (rrule) {
+			recur = icalproperty_get_rrule(rrule);
+			calnote->occurrences = recur.count;
+			switch (recur.freq) {
+			case ICAL_SECONDLY_RECURRENCE:
+				dprintf("Not supported recurrence type. Approximating recurrence\n");
+				calnote->recurrence = recur.interval / 3600;
+				if (!calnote->recurrence)
+					calnote->recurrence = 1;
+				break;
+			case ICAL_MINUTELY_RECURRENCE:
+				dprintf("Not supported recurrence type. Approximating recurrence\n");
+				calnote->recurrence = recur.interval / 60;
+				if (!calnote->recurrence)
+					calnote->recurrence = 1;
+				break;
+			case ICAL_HOURLY_RECURRENCE:
+				calnote->recurrence = recur.interval;
+				break;
+			case ICAL_DAILY_RECURRENCE:
+				calnote->recurrence = recur.interval * 24;
+				break;
+			case ICAL_WEEKLY_RECURRENCE:
+				calnote->recurrence = recur.interval * 24 * 7;
+				break;
+			case ICAL_MONTHLY_RECURRENCE:
+				calnote->recurrence = GN_CALNOTE_MONTHLY;
+				break;
+			case ICAL_YEARLY_RECURRENCE:
+				calnote->recurrence = GN_CALNOTE_YEARLY;
+				break;
+			case ICAL_NO_RECURRENCE:
+				calnote->recurrence = GN_CALNOTE_NEVER;
+				break;
+			default:
+				dprintf("Not supported recurrence type. Assuming never\n");
+				calnote->recurrence = GN_CALNOTE_NEVER;
+				break;
+			}
+		}
+
 		str = icalcomponent_get_location(compresult);
 		if (!str)
 			str = "";
 		snprintf(calnote->mlocation, sizeof(calnote->mlocation), "%s", str);
-
-		dprintf("Component found\n%s\n", icalcomponent_as_ical_string(compresult));
 	}
-	if (compresult)
+	if (compresult) {
+#ifdef DEBUG
+		char *temp;
+
+		temp = icalcomponent_as_ical_string(compresult);
+		dprintf("Component found\n%s\n", temp);
+		free(temp);
+#endif
 		icalcomponent_free(compresult);
+	}
 
 	return retval;
 }
@@ -508,7 +607,7 @@ GNOKII_API gn_error gn_ical2calnote(FILE *f, gn_calnote *calnote, int id)
 #endif /* HAVE_LIBICAL */
 }
 
-/* read a vcalendar event given by id from file f and store the data in calnote */
+/* read a vcalendar event given by id from string ical and store the data in calnote */
 GNOKII_API gn_error gn_icalstr2calnote(const char * ical, gn_calnote *calnote, int id)
 {
 #ifdef HAVE_LIBICAL
@@ -621,7 +720,7 @@ static int gn_ical2todo_real(icalcomponent *comp, gn_todo *ctodo, int id)
 
 		/* summary goes into text */
 		/* TODO: UTF8 --> ascii */
-		snprintf(ctodo->text, GN_CALNOTE_MAX_LENGTH-1, "%s", icalcomponent_get_summary(compresult));
+		snprintf(ctodo->text, GN_TODO_MAX_LENGTH-1, "%s", icalcomponent_get_summary(compresult));
 
 		/* priority */
 		ctodo->priority = icalproperty_get_priority((const icalproperty *)priority);
