@@ -102,6 +102,10 @@ gn_error getcalendarnote(int argc, char *argv[], gn_data *data, struct gn_statem
 			return getcalendarnote_usage(stderr, -1);
 		}
 	}
+	if (argc > optind + 1) {
+		/* There are too many arguments that don't start with '-' */
+		return getcalendarnote_usage(stderr, -1);
+	}
 
 	for (i = first_location; i <= last_location; i++) {
 		memset(&calnote, 0, sizeof(calnote));
@@ -121,7 +125,8 @@ gn_error getcalendarnote(int argc, char *argv[], gn_data *data, struct gn_statem
 				ical = gn_calnote2icalstr(&calnote);
 				fprintf(stdout, "%s\n", ical);
 				free (ical);
-			} else {  /* plaint text output */
+			} else {  /* Plain text output */
+				/* Translators: this is the header of a calendar note. Example: 1 (1). Type: Meeting */
 				fprintf(stdout, _("%d (%d). %s: %s\n"), i, calnote.location, _("Type"), gn_calnote_type2str(calnote.type));
 
 				fprintf(stdout, _("   Start date: %d-%02d-%02d\n"), calnote.time.year,
@@ -332,6 +337,7 @@ gn_error deletecalendarnote(int argc, char *argv[], gn_data *data, struct gn_sta
 	gn_data_clear(data);
 	memset(&calnote, 0, sizeof(gn_calnote));
 	data->calnote = &calnote;
+	memset(&clist, 0, sizeof(gn_calnote_list));
 	data->calnote_list = &clist;
 
 	first_location = gnokii_atoi(optarg);
