@@ -1,6 +1,6 @@
 /*
 
-  $Id: gnokii-phonebook.c,v 1.23 2008/04/14 21:17:27 dforsi Exp $
+  $Id$
 
   G N O K I I
 
@@ -63,7 +63,7 @@ void phonebook_usage(FILE *f)
 }
 
 /* Displays usage of --getphonebook command */
-void getphonebook_usage(FILE *f, int exitval)
+int getphonebook_usage(FILE *f, int exitval)
 {
 	fprintf(f, _(" usage: --getphonebook memory start [end]  reads phonebook entries from memory type\n"
 			"                                         (SM, ME, IN, OU, ...) messages starting\n"
@@ -79,7 +79,7 @@ void getphonebook_usage(FILE *f, int exitval)
 			"       --ldif                            output in ldif format\n"
 			"\n"
 		));
-	exit(exitval);
+	return exitval;
 }
 
 /* Get requested range of memory storage entries and output to stdout in
@@ -91,7 +91,6 @@ gn_error getphonebook(int argc, char *argv[], gn_data *data, struct gn_statemach
 	int i, count, start_entry, end_entry, num_entries = INT_MAX, explicit_end = true;
 	gn_error error = GN_ERR_NONE;
 	char *memory_type_string;
-	char location[32];
 	int type = 0; /* Output type:
 				0 - not formatted
 				1 - CSV
@@ -115,10 +114,10 @@ gn_error getphonebook(int argc, char *argv[], gn_data *data, struct gn_statemach
 
 	start_entry = gnokii_atoi(argv[optind]);
 	if (errno || start_entry < 0)
-		getphonebook_usage(stderr, -1);
+		return getphonebook_usage(stderr, -1);
 	end_entry = parse_end_value_option(argc, argv, optind + 1, start_entry);
 	if (errno || end_entry < 0)
-		getphonebook_usage(stderr, -1);
+		return getphonebook_usage(stderr, -1);
 
 	i = getopt_long(argc, argv, "rvl", options, NULL);
 	switch (i) {
@@ -135,11 +134,11 @@ gn_error getphonebook(int argc, char *argv[], gn_data *data, struct gn_statemach
 		/* default */
 		break;
 	default:
-		getphonebook_usage(stderr, -1);
+		return getphonebook_usage(stderr, -1);
 	}
 	if (argc - optind > 2) {
 		/* There are too many arguments that don't start with '-' */
-		getphonebook_usage(stderr, -1);
+		return getphonebook_usage(stderr, -1);
 	}
 
 	if (end_entry == INT_MAX) {
@@ -173,9 +172,13 @@ gn_error getphonebook(int argc, char *argv[], gn_data *data, struct gn_statemach
 			case 1:
 				gn_file_phonebook_raw_write(stdout, &entry, memory_type_string);
 				break;
-			case 2:
-				snprintf(location, sizeof(location), "%s%d", memory_type_string, entry.location);
-				gn_phonebook2vcard(stdout, &entry, location);
+			case 2: {
+					char *s;
+
+					s = gn_phonebook2vcardstr(&entry);
+					fprintf (stdout, "%s", s);
+					free (s);
+				}
 				break;
 			case 3:
 				gn_phonebook2ldif(stdout, &entry);
@@ -222,6 +225,9 @@ gn_error getphonebook(int argc, char *argv[], gn_data *data, struct gn_statemach
 				for (i = 0; i < entry.subentries_count; i++) {
 					fprintf(stdout, "%s: ", gn_subentrytype2string(entry.subentries[i].entry_type, entry.subentries[i].number_type));
 					switch (entry.subentries[i].entry_type) {
+					case GN_PHONEBOOK_ENTRY_ExtGroup:
+					        fprintf(stdout, "%d", entry.subentries[i].data.id);
+					        break;
 					case GN_PHONEBOOK_ENTRY_Birthday:
 					case GN_PHONEBOOK_ENTRY_Date:
 						fprintf(stdout, _("%04u.%02u.%02u %02u:%02u:%02u"), entry.subentries[i].data.date.year, entry.subentries[i].data.date.month, entry.subentries[i].data.date.day, entry.subentries[i].data.date.hour, entry.subentries[i].data.date.minute, entry.subentries[i].data.date.second);
@@ -261,7 +267,7 @@ gn_error getphonebook(int argc, char *argv[], gn_data *data, struct gn_statemach
 }
 
 /* Displays usage of --getphonebook command */
-void writephonebook_usage(FILE *f, int exitval)
+int writephonebook_usage(FILE *f, int exitval)
 {
 	fprintf(f, _(" usage:  --writephonebook [[-o|--overwrite]|[-f|--find-free]]\n"
 		     "                 [-m|--memory-type|--memory memory_type]\n"
@@ -269,7 +275,7 @@ void writephonebook_usage(FILE *f, int exitval)
 		     "                 [[-v|--vcard]|[-l|--ldif]]\n"
 		     "\n"
 		));
-	exit(exitval);
+	return exitval;
 }
 
 /* Read data from stdin, parse and write to phone.  The parsing is relatively
@@ -311,12 +317,12 @@ gn_error writephonebook(int argc, char *argv[], gn_data *data, struct gn_statema
 			break;
 		case 'v':
 			if (type)
-				writephonebook_usage(stderr, -1);
+				return writephonebook_usage(stderr, -1);
 			type = 1;
 			break;
 		case 'l':
 			if (type)
-				writephonebook_usage(stderr, -1);
+				return writephonebook_usage(stderr, -1);
 			type = 2;
 			break;
 		case 'f':
@@ -328,22 +334,21 @@ gn_error writephonebook(int argc, char *argv[], gn_data *data, struct gn_statema
 		case 'n':
 			default_location = gnokii_atoi(optarg);
 			if (errno || default_location < 0)
-				writephonebook_usage(stderr, -1);
+				return writephonebook_usage(stderr, -1);
 			break;
 		default:
-			writephonebook_usage(stderr, -1);
-			break;
+			return writephonebook_usage(stderr, -1);
 		}
 	}
 	if (argc > optind) {
 		/* There are too many arguments that don't start with '-' */
-		writephonebook_usage(stderr, -1);
+		return writephonebook_usage(stderr, -1);
 	}
 
 	line = oline;
 
 	/* Go through data from stdin. */
-	while (1) {
+	while (!feof(stdin)) {
 		error = GN_ERR_NONE;
 
 		memset(&entry, 0, sizeof(gn_phonebook_entry));
@@ -406,7 +411,7 @@ gn_error writephonebook(int argc, char *argv[], gn_data *data, struct gn_statema
 
 			if (error == GN_ERR_NONE || error == GN_ERR_EMPTYLOCATION) {
 				if (!aux.empty && error != GN_ERR_EMPTYLOCATION) {
-					char ans[8];
+					char ans[8] = "";
 
 					fprintf(stdout, _("Location busy. "));
 					confirm = -1;
@@ -445,7 +450,7 @@ gn_error writephonebook(int argc, char *argv[], gn_data *data, struct gn_statema
 			 * location */
 			default_location++;
 		} else
-			fprintf(stderr, _("Write FAILED (%s): memory type: %s, loc: %d, name: %s, number: %s\n"), 
+			fprintf(stderr, _("Write failed (%s): memory type: %s, loc: %d, name: %s, number: %s\n"),
 				 gn_error_print(error), gn_memory_type2str(entry.memory_type), entry.location, entry.name, entry.number);
 	}
 out:
@@ -456,12 +461,12 @@ out:
 }
 
 /* Displays usage of --deletephonebook command */
-void deletephonebook_usage(FILE *f, int exitval)
+int deletephonebook_usage(FILE *f, int exitval)
 {
 	fprintf(f, _(" usage: --deletephonebook memory_type start_number [end_number|end]\n"
 			"\n"
 		));
-	exit(exitval);
+	return exitval;
 }
 
 /* Delete phonebook entry */
@@ -473,7 +478,7 @@ gn_error deletephonebook(int argc, char *argv[], gn_data *data, struct gn_statem
 	int i, first_location, last_location;
 
 	if (argc < 3)
-		deletephonebook_usage(stderr, -1);
+		return deletephonebook_usage(stderr, -1);
 
 	/* Handle command line args that set memory type and location. */
 	memory_type_string = optarg;
@@ -485,10 +490,10 @@ gn_error deletephonebook(int argc, char *argv[], gn_data *data, struct gn_statem
 
 	first_location = gnokii_atoi(argv[optind]);
 	if (errno || first_location < 0)
-		deletephonebook_usage(stderr, -1);
+		return deletephonebook_usage(stderr, -1);
 	last_location = parse_end_value_option(argc, argv, optind + 1, first_location);
 	if (errno || last_location < 0)
-		deletephonebook_usage(stderr, -1);
+		return deletephonebook_usage(stderr, -1);
 
 	for (i = first_location; i <= last_location; i++) {
 		entry.location = i;
