@@ -111,6 +111,10 @@
 #  include <termios.h>
 #endif
 
+#ifdef HAVE_DIRENT_H
+#  include <dirent.h>
+#endif
+
 #if !defined(INT_MAX)
 #  define INT_MAX 2147483647
 #endif
@@ -247,7 +251,7 @@ time_t timegm(struct tm *tm);
 #endif
 
 /* Get rid of long defines. Use #if __unices__ */
-#define __unices__ defined(__svr4__) || defined(__FreeBSD__) || defined(__bsdi__) || defined(__MACH__) || defined(__OpenBSD__) || defined(__NetBSD__)
+#define __unices__ defined(__svr4__) || defined(__FreeBSD__) || defined(__bsdi__) || defined(__MACH__) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(__HAIKU__)
 
 /* This one is for NLS. */
 #ifdef ENABLE_NLS
@@ -320,7 +324,7 @@ time_t timegm(struct tm *tm);
 #    ifdef __cplusplus
 	typedef bool _Bool;
 #    else
-	typedef unsigned char _Bool;
+#define _Bool signed char
 #    endif
 #  endif
 #  define bool _Bool
@@ -329,4 +333,8 @@ time_t timegm(struct tm *tm);
 #  define __bool_true_false_are_defined 1
 #endif
 
+#ifdef __HAIKU__
+#  define FIOASYNC FIONBIO
+#endif
+
 #endif
