@@ -1,6 +1,6 @@
 /*
 
-  $Id: at-emulator.h,v 1.12 2004/09/20 16:05:38 bozo Exp $
+  $Id$
 
   G N O K I I
 
@@ -22,7 +22,7 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 1999, 2000 Hugh Blemings & Pavel Jank ml.
+  Copyright (C) 1999, 2000 Hugh Blemings & Pavel Janik ml.
 
   Header file for AT emulator code.
 
@@ -31,7 +31,6 @@
 #ifndef _gnokii_data_at_emulator_h
 #define _gnokii_data_at_emulator_h
 
-#include "config.h"
 #include "compat.h"
 #include "gnokii.h"
 
@@ -47,7 +46,7 @@ bool	gn_atem_command_plusc(char **buf);
 bool	gn_atem_command_plusg(char **buf);
 int	gn_atem_num_get(char **p);
 void	gn_atem_modem_result(int code);
-void    gn_atem_call_passup(gn_call_status call_status, gn_call_info *call_info, struct gn_statemachine *state);
+void    gn_atem_call_passup(gn_call_status call_status, gn_call_info *call_info, struct gn_statemachine *state, void *callback_data);
 void	gn_atem_cid_out(gn_call_info *callinfo);
 bool	gn_atem_command_diesis(char **buf);
 
