@@ -1,6 +1,6 @@
 /*
 
-  $Id: datapump.h,v 1.12 2003/03/06 21:31:08 pkot Exp $
+  $Id$
 
   G N O K I I
 
@@ -22,7 +22,7 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 1999, 2000 Hugh Blemings & Pavel Jank ml.
+  Copyright (C) 1999, 2000 Hugh Blemings & Pavel Janik ml.
 
   Header file for data pump code.
 
@@ -31,12 +31,11 @@
 #ifndef _gnokii_data_datapump_h
 #define _gnokii_data_datapump_h
 
-#include "config.h"
 #include "compat.h"
 #include "gnokii.h"
 
 /* Prototypes */
 bool	dp_Initialise(int read_fd, int write_fd);
-void    dp_CallPassup(gn_call_status call_status, gn_call_info *call_info, struct gn_statemachine *state);
+void    dp_CallPassup(gn_call_status call_status, gn_call_info *call_info, struct gn_statemachine *state, void *callback_data);
 
 #endif	/* _gnokii_data_datapump_h */
