@@ -1,7 +1,5 @@
 /*
 
-  $Id: gnokii-internal.h,v 1.39 2008/04/12 22:44:33 dforsi Exp $
-
   G N O K I I
 
   A Linux/Unix toolset and driver for the mobile phones.
@@ -23,7 +21,8 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
   Copyright (C) 1999-2000 Hugh Blemings, Pavel Janik
-  Copyright (C) 2000-2004 Pawel Kot, BORBELY Zoltan
+  Copyright (C) 2000-2004 BORBELY Zoltan
+  Copyright (C) 2000-2011 Pawel Kot
   Copyright (C) 2000      Marcin Wiacek, Chris Kemp
   Copyright (C) 2002      Ladis Michl
   Copyright (C) 2003      Osma Suominen, Martin Goldhahn
@@ -44,6 +43,14 @@
 
 #include "gnokii.h"
 
+#define return_on_error(func) \
+do { \
+	gn_error error; \
+	error = func; \
+	if (error != GN_ERR_NONE) \
+		return error; \
+} while (0)
+
 /* SMS */
 gn_error gn_sms_parse(gn_data *data);
 gn_error gn_sms_pdu2raw(gn_sms_raw *rawsms, unsigned char *pdu, int pdu_len, int flags);
@@ -81,7 +88,7 @@ int char_mblen(const char *src);
 unsigned int char_unicode_decode(unsigned char* dest, const unsigned char* src, int len);
 unsigned int char_unicode_encode(unsigned char* dest, const unsigned char* src, int len);
 
-void char_ascii_decode(unsigned char* dest, const unsigned char* src, int len);
+int char_default_alphabet_decode(unsigned char* dest, const unsigned char* src, int len);
 size_t char_ascii_encode(char *dest, size_t dest_len, const char *src, size_t len);
 
 void char_hex_decode(unsigned char* dest, const unsigned char* src, int len);
@@ -90,8 +97,8 @@ size_t char_hex_encode(char *dest, size_t dest_len, const char *src, size_t len)
 void char_ucs2_decode(unsigned char* dest, const unsigned char* src, int len);
 size_t char_ucs2_encode(char *dest, size_t dest_len, const char *src, size_t len);
 
-unsigned char char_def_alphabet_encode(unsigned char value);
-unsigned char char_def_alphabet_decode(unsigned char value);
+unsigned char char_def_alphabet_encode(unsigned int value);
+unsigned int char_def_alphabet_decode(unsigned char value);
 
 size_t char_uni_alphabet_encode(const char *value, size_t n, wchar_t *dest, MBSTATE *mbs);
 int char_uni_alphabet_decode(wchar_t value, unsigned char *dest, MBSTATE *mbs);
@@ -99,6 +106,14 @@ int char_uni_alphabet_decode(wchar_t value, unsigned char *dest, MBSTATE *mbs);
 extern char *char_bcd_number_get(u8 *number);
 extern int char_semi_octet_pack(char *number, unsigned char *output, gn_gsm_number_type type);
 
+int ucs2_encode(char *outstring, int outlen, const char *instring, int inlen);
+
+int char_def_alphabet_ext_count(unsigned char *input, int length);
+bool char_def_alphabet_ext(unsigned int value);
+
+int char_def_alphabet_string_copy(char *dest, const char *src, int len, int offset);
+gn_sms_dcs_alphabet_type char_def_alphabet_string_stats(char *str, int *enc_chars, int *ext_chars);
+
 /* Ringtones */
 int vcal_time_get(gn_timestamp *dt, char *time);
 int calnote_fill(gn_calnote *note, char *type, char *text, char *desc,
@@ -157,6 +172,8 @@ typedef void (*cfg_foreach_func)(const char *section, const char *key, const cha
 void cfg_foreach(const char *section, cfg_foreach_func func);
 char *cfg_set(struct gn_cfg_header *cfg, const char *section, const char *key, const char *value);
 int cfg_file_write(struct gn_cfg_header *cfg, const char *filename);
+/* Get some information about the given phone */
+gn_phone_model *gn_cfg_get_phone_model(struct gn_cfg_header *cfg, const char *product_name);
 
 gn_error isdn_cause2gn_error(char **src, char **msg, unsigned char loc, unsigned char cause);
 
@@ -171,6 +188,6 @@ int utf8_base64_decode(char *dest, int destlen, const char *src, int inlen);
 int utf8_base64_encode(char *dest, int destlen, const char *src, int inlen);
 
 int add_slashes(char *dest, char *src, int maxlen, int len);
-int strip_slashes(char *dest, char *src, int maxlen, int len);
+int strip_slashes(char *dest, const char *src, int maxlen, int len);
 
 #endif /* _gnokii_internal_h */
