@@ -1,6 +1,6 @@
 /*
 
-  $Id: statemachine.h,v 1.20 2005/07/24 18:54:57 pkot Exp $
+  $Id$
 
   G N O K I I
 
@@ -73,6 +73,7 @@ struct gn_statemachine {
 	gn_device device;
 	gn_link link;
 	gn_driver driver;
+	char *lockfile;
 	
 	/* Store last message for resend purposes */
 	unsigned char last_msg_type;
@@ -86,11 +87,18 @@ struct gn_statemachine {
 	gn_error response_error[GN_SM_WAITINGFOR_MAX_NUMBER];
 	/* Data structure to be filled in with the response */
 	gn_data *data[GN_SM_WAITINGFOR_MAX_NUMBER];
+
+	/* libfunctions internal data structure */
+	gn_error lasterror;
+	gn_data sm_data;
+	union {
+		gn_phonebook_entry pb_entry;
+	} u;
 };
 
-API gn_state gn_sm_loop(int timeout, struct gn_statemachine *state);
+GNOKII_API gn_state gn_sm_loop(int timeout, struct gn_statemachine *state);
 
 /* General way to call any driver function */
-API gn_error gn_sm_functions(gn_operation op, gn_data *data, struct gn_statemachine *sm);
+GNOKII_API gn_error gn_sm_functions(gn_operation op, gn_data *data, struct gn_statemachine *sm);
 
 #endif	/* _gnokii_statemachine_h */
