@@ -1,7 +1,5 @@
 /*
 
-  $Id$
-
   G N O K I I
 
   A Linux/Unix toolset and driver for the mobile phones.
@@ -121,8 +119,12 @@ typedef enum {
 #define NK6510_SUBCAL_NOTE_RCVD		0x1a	/* Received note */
 #define NK6510_SUBCAL_FREEPOS_RCVD	0x32	/* Received first free position */
 #define NK6510_SUBCAL_INFO_RCVD		0x3b	/* Received calendar summary */
+#define NK6510_SUBCAL_DEL_NOTE2_RESP	0x70	/* Delete note response */
 #define NK6510_SUBCAL_NOTE2_RCVD	0x7e	/* Received note (with more details) */
 #define NK6510_SUBCAL_ADD_NOTE_RESP	0x66	/* Add calendar note response */
+#define NK6510_SUBCAL_INFO2_RCVD	0x9f	/* Received calendar summary */
+#define NK6510_SUBCAL_FREEPOS2_RCVD	0x96	/* Received first free position */
+#define NK6510_SUBCAL_UNSUPPORTED	0xf0	/* Error code for unsupported FBUS frames */
 
 /* Calendar note types */
 #define NK6510_NOTE_REMINDER		0x00	/* Reminder */
@@ -131,16 +133,8 @@ typedef enum {
 #define NK6510_NOTE_BIRTHDAY		0x04	/* Birthday */
 #define NK6510_NOTE_MEMO		0x08	/* Memo */
 
-/* Phone Memory types */
-#define NK6510_MEMORY_DIALLED		0x01	/* Dialled numbers */
-#define NK6510_MEMORY_MISSED		0x02	/* Missed calls */
-#define NK6510_MEMORY_RECEIVED		0x03	/* Received calls */
-#define NK6510_MEMORY_PHONE		0x05	/* Telephone phonebook */
-#define NK6510_MEMORY_SIM		0x06	/* SIM phonebook */
-#define NK6510_MEMORY_SPEEDDIALS	0x0e	/* Speed dials */
-#define NK6510_MEMORY_GROUPS		0x10	/* Caller groups */
-
-#define NK6510_MEMORY_DC		0x01	/* ME dialed calls list */
+/* Phonebook Memory types */
+#define NK6510_MEMORY_DC		0x01	/* ME dialled calls list */
 #define NK6510_MEMORY_MC		0x02	/* ME missed (unanswered received) calls list */
 #define NK6510_MEMORY_RC		0x03	/* ME received calls list */
 #define NK6510_MEMORY_FD		0x04	/* ?? SIM fixdialling-phonebook */
@@ -150,6 +144,11 @@ typedef enum {
 #define NK6510_MEMORY_EN		0x08	/* ?? SIM (or ME) emergency number */
 #define NK6510_MEMORY_MT		0x09	/* ?? combined ME and SIM phonebook */
 #define NK6510_MEMORY_VOICE		0x0b	/* Voice Mailbox */
+#define NK6510_MEMORY_SPEEDDIALS	0x0e	/* Speed dials */
+#define NK6510_MEMORY_GROUPS		0x10	/* Caller groups */
+#define NK6510_MEMORY_EXTGROUPS		0x23	/* Extended Caller Groups */
+
+/* SMS Memory types */
 #define NK6510_MEMORY_IN		0x02	/* INBOX */
 #define NK6510_MEMORY_OU		0x03	/* OUTBOX */
 #define NK6510_MEMORY_AR		0x04	/* ARCHIVE */
@@ -174,6 +173,8 @@ typedef enum {
 #define NK6510_MEMORY_F18		0x23
 #define NK6510_MEMORY_F19		0x24
 #define NK6510_MEMORY_F20		0x25	/* MY FOLDERS 20 */
+#define NK6510_MEMORY_OUS		0x1a
+
 /* This is used when the memory type is unknown. */
 #define NK6510_MEMORY_XX		0xff
 
@@ -205,7 +206,7 @@ typedef enum {
  *   - get file list
  *   - filter sms files by name
  *   - get file
- * It is repeated for every SMS. As 'get file list' is the longest opration
+ * It is repeated for every SMS. As 'get file list' is the longest operation
  * it makes sense to cache the results. Define here for how long (seconds)
  * data is cached.
  */
