@@ -1,7 +1,5 @@
 /*
 
-  $Id: db.h,v 1.12 2005/04/26 06:34:39 ja Exp $
-
   S M S D
 
   A Linux/Unix tool for the mobile phones.
@@ -23,7 +21,7 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
   Copyright (C) 1999 Pavel Jank ml., Hugh Blemings
-  Copyright (C) 1999-2005 Jan Derfinak
+  Copyright (C) 1999-2011 Jan Derfinak
 
 */
 
@@ -34,10 +32,10 @@
 #include "smsd.h"
 #include "gnokii.h"
 
-extern void (*DB_Bye) (void);
-extern gint (*DB_ConnectInbox) (const DBConfig);
-extern gint (*DB_ConnectOutbox) (const DBConfig);
-extern gint (*DB_InsertSMS) (const gn_sms * const, const gchar * const);
-extern void (*DB_Look) (const gchar * const);
+GNOKII_API void (*DB_Bye) (void);
+GNOKII_API gint (*DB_ConnectInbox) (const DBConfig);
+GNOKII_API gint (*DB_ConnectOutbox) (const DBConfig);
+GNOKII_API gint (*DB_InsertSMS) (const gn_sms * const, const gchar * const);
+GNOKII_API gint (*DB_Look) (const gchar * const);
 
 #endif
