@@ -1,7 +1,5 @@
 /*
 
-  $Id: file.c,v 1.17 2006/10/03 21:26:38 pkot Exp $
-
   S M S D
 
   A Linux/Unix tool for the mobile phones.
@@ -22,7 +20,7 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 2002-2005 Jan Derfinak
+  Copyright (C) 2002-2011 Jan Derfinak
   
   This file is a module to smsd for file access.
 
@@ -126,11 +124,11 @@ GNOKII_API gint DB_InsertSMS (const gn_sms * const data, const gchar * const pho
   else
   {
     buf = g_string_sized_new (256);
-    g_string_sprintf (buf, "%s %s \"%02d-%02d-%02d %02d:%02d:%02d\"", \
-                      action, data->remote.number, data->smsc_time.year, \
-                      data->smsc_time.month, data->smsc_time.day, \
-                      data->smsc_time.hour, data->smsc_time.minute, \
-                      data->smsc_time.second);
+    g_string_printf (buf, "%s %s \"%02d-%02d-%02d %02d:%02d:%02d\"", \
+                     action, data->remote.number, data->smsc_time.year, \
+                     data->smsc_time.month, data->smsc_time.day, \
+                     data->smsc_time.hour, data->smsc_time.minute, \
+                     data->smsc_time.second);
     if ((p = popen (buf->str, "w")) == NULL)
     {
       g_free (text);
@@ -150,22 +148,23 @@ GNOKII_API gint DB_InsertSMS (const gn_sms * const data, const gchar * const pho
 }
 
 
-GNOKII_API void DB_Look (const gchar * const phone)
+GNOKII_API gint DB_Look (const gchar * const phone)
 {
   DIR *dir;
   struct dirent *dirent;
   FILE *smsFile;
   GString *buf;
   gint numError, error;
+  gint empty = 1;
 
 
   if (spool[0] == '\0')  // if user don't set spool dir, sending is disabled
-    return;
+    return (SMSD_NOK);
     
   if ((dir = opendir (spool)) == NULL)
   {
     g_print (_("Cannot open directory %s\n"), spool);
-    return;
+    return (SMSD_NOK);
   }
 
   buf = g_string_sized_new (64);
@@ -173,12 +172,13 @@ GNOKII_API void DB_Look (const gchar * const phone)
   while ((dirent = readdir (dir)))
   {
     gn_sms sms;
+    gint slen = 0;
     
     if (strcmp (dirent->d_name, ".") == 0 || strcmp (dirent->d_name, "..") == 0 ||
         strncmp (dirent->d_name, "ERR.", 4) == 0)
       continue;
     
-    g_string_sprintf (buf, "%s/%s", spool, dirent->d_name);
+    g_string_printf (buf, "%s/%s", spool, dirent->d_name);
     
     if ((smsFile = fopen (buf->str, "r")) == NULL)
     {
@@ -186,16 +186,29 @@ GNOKII_API void DB_Look (const gchar * const phone)
       continue;
     }
     
+    empty = 0;
     gn_sms_default_submit (&sms);
     memset (&sms.remote.number, 0, sizeof (sms.remote.number));
 
-    fgets (sms.remote.number, sizeof (sms.remote.number), smsFile);
-    if (sms.remote.number[strlen (sms.remote.number) - 1] == '\n')
-      sms.remote.number[strlen (sms.remote.number) - 1] = '\0';
+    if (fgets (sms.remote.number, sizeof (sms.remote.number), smsFile))
+      slen = strlen (sms.remote.number);
+    if (slen < 1)
+    {
+      error = -1;
+      fclose (smsFile);
+      g_print (_("Remote number is empty in %s!\n"), buf->str);
+      goto handle_file;
+    }
     
-    fgets ((gchar *) sms.user_data[0].u.text, GN_SMS_MAX_LENGTH + 1, smsFile);
-    if (sms.user_data[0].u.text[strlen ((gchar *) sms.user_data[0].u.text) - 1] == '\n')
-      sms.user_data[0].u.text[strlen ((gchar *) sms.user_data[0].u.text) - 1] = '\0';
+    if (sms.remote.number[slen - 1] == '\n')
+      sms.remote.number[slen - 1] = '\0';
+    
+    /* Initialize SMS text */
+    memset (&sms.user_data[0].u.text, 0, sizeof (sms.user_data[0].u.text));
+    
+    slen = fread ((gchar *) sms.user_data[0].u.text, 1, GN_SMS_MAX_LENGTH, smsFile);
+    if (slen > 0 && sms.user_data[0].u.text[slen - 1] == '\n')
+      sms.user_data[0].u.text[slen - 1] = '\0';
      
     fclose (smsFile);
     
@@ -223,6 +236,7 @@ GNOKII_API void DB_Look (const gchar * const phone)
     }
     while ((error == GN_ERR_TIMEOUT || error == GN_ERR_FAILED) && numError++ < 3);
 
+ handle_file:
     if (error == GN_ERR_NONE)
     {
       if (unlink (buf->str))
@@ -233,7 +247,7 @@ GNOKII_API void DB_Look (const gchar * const phone)
       GString *buf2;
       
       buf2 = g_string_sized_new (64);
-      g_string_sprintf (buf2, "%s/ERR.%s", spool, dirent->d_name);
+      g_string_printf (buf2, "%s/ERR.%s", spool, dirent->d_name);
       
       g_print (_("Cannot send sms from file %s\n"), buf->str);
       if (rename (buf->str, buf2->str))
@@ -249,4 +263,9 @@ GNOKII_API void DB_Look (const gchar * const phone)
   
   g_string_free (buf, TRUE);
   closedir (dir);
+
+  if (empty)
+    return (SMSD_OUTBOXEMPTY);
+  else
+    return (SMSD_OK);
 }
