@@ -1,7 +1,5 @@
 /*
 
-  $Id: lowlevel.c,v 1.53 2008/07/27 16:47:44 pkot Exp $
-
   S M S D
 
   A Linux/Unix tool for the mobile phones.
@@ -22,8 +20,8 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 1999      Pavel Jank ml., Hugh Blemings
-  Copyright (C) 1999-2004 Jan Derfinak
+  Copyright (C) 1999 Pavel Jank ml., Hugh Blemings
+  Copyright (C) 1999-2011 Jan Derfinak
 
 */
 
@@ -78,47 +76,25 @@ inline static PhoneEvent *RemoveEvent (void)
 
 static gn_error InitModelInf (void)
 {
-  gn_data data;
-  gn_error error;
-  char *aux;
-  char model[GN_MODEL_MAX_LENGTH], rev[GN_REVISION_MAX_LENGTH], 
-	manufacturer[GN_MANUFACTURER_MAX_LENGTH];
+  phoneMonitor.phone.version = gn_lib_get_phone_product_name (sm);
 
-  data.manufacturer = manufacturer;
-  data.model = model;
-  data.revision = rev;
-                          
-  error = gn_sm_functions (GN_OP_GetModel, &data, sm);
-  if (error != GN_ERR_NONE)
-  {
-    return error;
-  }
-    
-  g_free (phoneMonitor.phone.version);
-  phoneMonitor.phone.version = g_strdup (model);
+  phoneMonitor.phone.model = gn_lib_get_phone_model (sm);
 
-  g_free (phoneMonitor.phone.model);
-  aux = (char *)gn_phone_model_get(model)->model;
-  if (aux == NULL)
-    phoneMonitor.phone.model = g_strdup (_("unknown"));
-  else
-    phoneMonitor.phone.model = g_strdup (aux);
+  phoneMonitor.supported = gn_phone_model_get (phoneMonitor.phone.model)->flags;
 
-  phoneMonitor.supported = gn_phone_model_get (model)->flags;
+  phoneMonitor.phone.revision = gn_lib_get_phone_revision (sm);
 
-  g_free (phoneMonitor.phone.revision);
-  phoneMonitor.phone.revision = g_strdup (rev);
-
-  gn_log_xdebug ("Version: %s\n", phoneMonitor.phone.version);
-  gn_log_xdebug ("Model: %s\n", phoneMonitor.phone.model);
-  gn_log_xdebug ("Revision: %s\n", phoneMonitor.phone.revision);
+  gn_log_xdebug ("Model        : %s\n", gn_lib_get_phone_model (sm));
+  gn_log_xdebug ("Product name : %s\n", gn_lib_get_phone_product_name (sm));
+  gn_log_xdebug ("Revision     : %s\n", gn_lib_get_phone_revision (sm));
 
   return GN_ERR_NONE;
 }
 
 static void busterminate (void)
 {
-  if (sm) {
+  if (sm)
+  {
     gn_lib_phone_close(sm);
     gn_lib_phoneprofile_free(&sm);
   }
@@ -132,8 +108,8 @@ static gn_error fbusinit (const char * const iname)
   error = gn_lib_phoneprofile_load(iname, &sm);
   if (error != GN_ERR_NONE)
   {
-    g_print (_("Cannot load phone %s!\nDo you have proper section in gnokiirc?\n"), iname);
-    g_print (_("%s\n"), gn_error_print(error));
+    g_print (_("Cannot load phone %s!\nDo you have proper section in the config file?\n"), iname);
+    g_print (_("Error: %s\n"), gn_error_print (error));
     exit (-1);
   }
 
@@ -146,7 +122,7 @@ static gn_error fbusinit (const char * const iname)
   if (error != GN_ERR_NONE)
   {
     g_print (_("GSM/FBUS init failed! (Unknown model?). Quitting.\n"));
-    g_print (_("%s\n"), gn_error_print(error));
+    g_print (_("Error: %s\n"), gn_error_print (error));
     return (error);
   }
 
@@ -162,9 +138,9 @@ static gn_error fbusinit (const char * const iname)
 
 void InitPhoneMonitor (void)
 {
-  phoneMonitor.phone.model = g_strdup (_("unknown"));
-  phoneMonitor.phone.version = g_strdup (_("unknown"));
-  phoneMonitor.phone.revision = g_strdup (_("unknown"));
+  phoneMonitor.phone.model = _("unknown");
+  phoneMonitor.phone.version = _("unknown");
+  phoneMonitor.phone.revision = _("unknown");
   phoneMonitor.supported = 0;
 //  phoneMonitor.working = FALSE;
 //  phoneMonitor.sms.unRead = 0;
@@ -208,7 +184,7 @@ static void RefreshSMS (const gint number)
   if (number < 1)
   {
     gn_log_xdebug ("RefreshSMS is started with number of sms less than 1.\n\
-This should not happen.\nSkiping.");
+This should not happen.\nSkipping.");
     return;
   }
   
@@ -222,26 +198,6 @@ This should not happen.\nSkiping.");
   folder.folder_id = 0;
   data.sms_folder = &folder;
 
-  if (smsdConfig.firstSMS < 0)
-  {
-    gn_log_xdebug ("Detecting first SMS position.\n");
-    
-    msg = g_malloc (sizeof (gn_sms));
-    memset (msg, 0, sizeof (gn_sms));
-    msg->memory_type = smsdConfig.memoryType;
-    msg->number = 0;
-    data.sms = msg;
-
-    if ((error = gn_sms_get (&data, sm)) == GN_ERR_INVALIDLOCATION)
-      smsdConfig.firstSMS = 1;
-    else
-      smsdConfig.firstSMS = 0;
-      
-    gn_log_xdebug ("First SMS position is %d\n", smsdConfig.firstSMS);
-    
-    g_free (msg);
-  }
-    
   i = smsdConfig.firstSMS;
   while (1)
   {
@@ -250,13 +206,15 @@ This should not happen.\nSkiping.");
     msg->memory_type = smsdConfig.memoryType;
     msg->number = i++;
     data.sms = msg;
-    gn_log_xdebug("Reading SMS %d\n", i-1);
+    gn_log_xdebug ("Reading SMS %d of %d\n", i - 1, number);
     if ((error = gn_sms_get (&data, sm)) == GN_ERR_NONE)
     {
       phoneMonitor.sms.messages = g_slist_append (phoneMonitor.sms.messages, msg);
       phoneMonitor.sms.number++;
-      
-      if (phoneMonitor.sms.number == number)
+
+      gn_log_xdebug ("Message counter: %d\n", phoneMonitor.sms.number);
+
+      if (phoneMonitor.sms.number >= number)
       {
         pthread_cond_signal (&smsCond);
         return;
@@ -268,15 +226,15 @@ This should not happen.\nSkiping.");
      */
     else if (error == GN_ERR_INVALIDLOCATION && i > 1)
     {
-      gn_log_xdebug("Invalid location\n");
+      gn_log_xdebug ("gn_sms_get returned error %d: %s\n", error, gn_error_print (error));
       g_free (msg);
       pthread_cond_signal (&smsCond);
       break;
     }
     else
     {
+      gn_log_xdebug ("gn_sms_get returned error %d: %s\n", error, gn_error_print (error));
       g_free (msg);
-      gn_log_xdebug ("gn_sms_get returned %d\n", error);
     }
 
     usleep (500000);
@@ -288,6 +246,7 @@ static gint A_SendSMSMessage (gpointer data)
 {
   D_SMSMessage *d = (D_SMSMessage *) data;
   gn_data *dt;
+  gn_error status;
 
   if (!d)
     return (GN_ERR_UNKNOWN);
@@ -296,14 +255,34 @@ static gint A_SendSMSMessage (gpointer data)
   dt = calloc (1, sizeof (gn_data));
   if (!d->sms->smsc.number[0])
   {
-    dt->message_center = calloc (1, sizeof (gn_sms_message_center));
-    dt->message_center->id = 1;
-    if (gn_sm_functions (GN_OP_GetSMSCenter, dt, sm) == GN_ERR_NONE)
+    /* keep the local cache of smsc information, it doesn't change too often */
+    static gn_gsm_number_type smsc_type = -1; /* -1: unititialized, -2: GetSMSCenter does not work */
+    static char smsc_number[GN_BCD_STRING_MAX_LENGTH];
+
+    if (smsc_type == -2)
+      status = GN_ERR_FAILED;
+    else
+      status = GN_ERR_NONE;
+
+    if (smsc_type == -1)
     {
-      strcpy (d->sms->smsc.number, dt->message_center->smsc.number);
-      d->sms->smsc.type = dt->message_center->smsc.type;
+      dt->message_center = calloc (1, sizeof (gn_sms_message_center));
+      dt->message_center->id = 1;
+      if ((status = gn_sm_functions (GN_OP_GetSMSCenter, dt, sm)) == GN_ERR_NONE)
+      {
+        strcpy (smsc_number, dt->message_center->smsc.number);
+        smsc_type = dt->message_center->smsc.type;
+      } else {
+        smsc_type = -2;
+      }
+      free (dt->message_center);
+      dt->message_center = NULL;
+    }
+    if (status == GN_ERR_NONE)
+    {
+        strcpy (d->sms->smsc.number, smsc_number);
+        d->sms->smsc.type = smsc_type;
     }
-    free (dt->message_center);
   }
   
   if (!d->sms->smsc.type)
@@ -312,11 +291,15 @@ static gint A_SendSMSMessage (gpointer data)
   gn_data_clear (dt);
   dt->sms = d->sms;
   d->status = gn_sms_send (dt, sm);
+  status = d->status;
+  /* we do not use sms reference numbers in smsd */
+  free (dt->sms->reference);
+  dt->sms->reference = NULL;
   free (dt);
   pthread_cond_signal (&sendSMSCond);
   pthread_mutex_unlock (&sendSMSMutex);
 
-  return (d->status);
+  return (status);
 }
 
 /*
@@ -334,7 +317,7 @@ static gint A_DeleteSMSMessage (gpointer data)
 
   dt = calloc (1, sizeof (gn_data));
   if (!dt) {
-    gn_log_xdebug("Failed to allocate memory.\n");
+    gn_log_xdebug ("Failed to allocate memory.\n");
     return GN_ERR_FAILED;
   }
   dt->sms = (gn_sms *) data;
@@ -343,19 +326,19 @@ static gint A_DeleteSMSMessage (gpointer data)
   dt->sms_folder_list = &SMSFolderList;
   if (dt->sms)
   {
-    gn_log_xdebug("Deleting SMS %d\n", dt->sms->number);
+    gn_log_xdebug ("Deleting SMS %d\n", dt->sms->number);
     /* FIXME: error handling */
     error = gn_sms_delete (dt, sm);
+    gn_log_xdebug ("gn_sms_delete returned error %d: %s\n", error, gn_error_print (error));
 
-    gn_log_xdebug("Error: %s\n", gn_error_print(error));
-    pthread_mutex_lock (&smsMutex);
+//    pthread_mutex_lock (&smsMutex);
     phoneMonitor.sms.messages = g_slist_remove (phoneMonitor.sms.messages, data);
     phoneMonitor.sms.number--;
     FreeElement (data, NULL);
-    pthread_mutex_unlock (&smsMutex);
-  } else {
-    gn_log_xdebug("Internal error: dt->sms == NULL\n");
-  }
+//    pthread_mutex_unlock (&smsMutex);
+  } 
+  else
+    gn_log_xdebug ("Internal error: dt->sms == NULL\n");
 
   free (dt);
   return (error);
@@ -383,6 +366,7 @@ static void RealConnect (void *phone)
   gn_sms_folder SMSFolder;
   PhoneEvent *event;
   gn_error error;
+  int consecutive_errors = 0;
 
   data = calloc (1, sizeof (gn_data));
   
@@ -405,25 +389,31 @@ static void RealConnect (void *phone)
 
   while (1)
   {
-//    phoneMonitor.working = FALSE;
-
     pthread_mutex_lock (&smsMutex);
     
+    /* The event queue must be processed before RefreshSMS ()! */
+    while ((event = RemoveEvent ()) != NULL)
+    {
+      gn_log_xdebug ("Processing Event: %d\n", event->event);
+      if (event->event <= Event_Exit)
+        if ((error = DoAction[event->event] (event->data)) != GN_ERR_NONE)
+          g_print (_("Event %d failed with return code %d!\n"), event->event, error);
+      g_free (event);
+    }
+  
     if (phoneMonitor.supported & PM_FOLDERS)
     {
       data->sms_folder = &SMSFolder;
       SMSFolder.folder_id = smsdConfig.memoryType;
       if ((error = gn_sm_functions (GN_OP_GetSMSFolderStatus, data, sm)) == GN_ERR_NONE)
       {
-        gn_log_xdebug ("GN_OP_GetSMSFolderStatus returned  %d\n",
+        gn_log_xdebug ("GN_OP_GetSMSFolderStatus returned (number) %d\n",
                        SMSFolder.number);
         gn_log_xdebug ("phoneMonitor.sms.number %d\n",
                        phoneMonitor.sms.number);
         if (phoneMonitor.sms.number != SMSFolder.number)
         {
-//          phoneMonitor.working = TRUE;
           RefreshSMS (SMSFolder.number);
-//          phoneMonitor.working = FALSE;
         }
 //        phoneMonitor.sms.unRead = 0;
       }
@@ -439,13 +429,12 @@ static void RealConnect (void *phone)
       {
         gn_log_xdebug ("GN_OP_GetSMSStatus returned (number, unread) %d, %d\n",
                        SMSStatus.number, SMSStatus.unread);
-        gn_log_xdebug ("phoneMonitor %d\n", phoneMonitor.sms.number);
+        gn_log_xdebug ("phoneMonitor.sms.number %d\n",
+                       phoneMonitor.sms.number);
         if (/* phoneMonitor.sms.unRead != SMSStatus.unread || */
             phoneMonitor.sms.number != SMSStatus.number)
         {
-//          phoneMonitor.working = TRUE;
           RefreshSMS (SMSStatus.number);
-//          phoneMonitor.working = FALSE;
         }
 //        phoneMonitor.sms.unRead = SMSStatus.unread;
       }
@@ -462,19 +451,21 @@ static void RealConnect (void *phone)
         break;
       }
       else
+      {
         g_print ("%s:%d error: %d, %s\n", __FILE__, __LINE__, error, gn_error_print(error));
+	consecutive_errors++;
+	if (consecutive_errors > MAX_CONSECUTIVE_ERRORS)
+	{
+	  g_print (_("Too many consecutive errors, restarting connection.\n"));
+	  break;
+	}
+      }
+    }
+    else
+    {
+      consecutive_errors = 0;
     }
 
-    while ((event = RemoveEvent ()) != NULL)
-    {
-      gn_log_xdebug ("Processing Event: %d\n", event->event);
-//      phoneMonitor.working = TRUE;
-      if (event->event <= Event_Exit)
-        if ((error = DoAction[event->event] (event->data)) != GN_ERR_NONE)
-          g_print (_("Event %d failed with return code %d!\n"), event->event, error);
-      g_free (event);
-    }
-    
     sleep (smsdConfig.refreshInt);
   }
 
