@@ -6,8 +6,7 @@ CREATE TABLE inbox (
   id int(10) unsigned NOT NULL auto_increment,
   number varchar(20) NOT NULL default '',
   smsdate datetime NOT NULL default '0000-00-00 00:00:00',
--- Warning (Code 1287): 'TIMESTAMP(14)' is deprecated; use 'TIMESTAMP' instead
-  insertdate timestamp(14) NOT NULL,
+  insertdate timestamp DEFAULT CURRENT_TIMESTAMP,
   text text,
   phone tinyint(4),
   processed tinyint(4) NOT NULL default '0',
@@ -16,12 +15,11 @@ CREATE TABLE inbox (
 
 CREATE TABLE outbox (
   id int(10) unsigned NOT NULL auto_increment,
-  number varchar(20) NOT NULL default '',
--- Warning (Code 1287): 'TIMESTAMP(14)' is deprecated; use 'TIMESTAMP' instead
-  processed_date timestamp(14) NOT NULL,
--- Warning (Code 1287): 'TIMESTAMP(14)' is deprecated; use 'TIMESTAMP' instead
-  insertdate timestamp(14) NOT NULL,
-  text varchar(160) default NULL,
+-- MySQL parses but ignores CHECK clause
+  number varchar(20) NOT NULL CHECK (number <> ''),
+  processed_date timestamp DEFAULT 0,
+  insertdate timestamp DEFAULT CURRENT_TIMESTAMP,
+  text text default NULL,
   phone tinyint(4),
   processed tinyint(4) NOT NULL default '0',
   error tinyint(4) NOT NULL default '-1',
@@ -35,14 +33,13 @@ CREATE TABLE multipartinbox (
   id int(10) unsigned NOT NULL auto_increment,
   number varchar(20) NOT NULL default '',
   smsdate datetime NOT NULL default '0000-00-00 00:00:00',
--- Warning (Code 1287): 'TIMESTAMP(14)' is deprecated; use 'TIMESTAMP' instead
-  insertdate timestamp(14) NOT NULL,
-  text text,
+  insertdate timestamp DEFAULT CURRENT_TIMESTAMP,
+  text varchar(160),
   phone tinyint(4),
   processed tinyint(4) NOT NULL default '0',
-  refnum int(8) default NULL,
-  maxnum int(8) default NULL,
-  curnum int(8) default NULL,
+  refnum smallint default NULL,
+  maxnum smallint default NULL,
+  curnum smallint default NULL,
   PRIMARY KEY  (id)
 );
 
