@@ -1,7 +1,5 @@
 /*
 
-  $Id$
-
   S M S D
 
   A Linux/Unix tool for the mobile phones.
@@ -23,21 +21,26 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
   Copyright (C) 1999 Pavel Jank ml., Hugh Blemings
-  Copyright (C) 1999-2005 Jan Derfinak
+  Copyright (C) 1999-2011 Jan Derfinak
 
 */
 
 #ifndef SMSD_H
 #define SMSD_H
 
-#include <glib.h>
+#include "config.h"
+#include "compat.h"
+
 #include "gnokii.h"
 
+#include <glib.h>
+
 /* error definition */
 #define SMSD_OK		0
 #define SMSD_NOK	1
 #define SMSD_DUPLICATE	2
 #define SMSD_WAITING	3
+#define SMSD_OUTBOXEMPTY	4
 
 typedef enum {
   SMSD_READ_REPORTS = 1
@@ -62,6 +65,7 @@ typedef struct {
   gchar *db;
   gchar *host;
   gchar *schema;
+  gchar *clientEncoding;
 } DBConfig;
 
 extern SmsdConfig smsdConfig;
