@@ -1,6 +1,6 @@
 /*
 
-  $Id: xgnokii.c,v 1.99 2008/06/05 16:50:18 dforsi Exp $
+  $Id$
   
   X G N O K I I
 
@@ -59,8 +59,6 @@
 #  include <signal.h>
 #else
 #  include <windows.h>
-#  include "../win32/winserial.h"
-#  define WRITEPHONE(a, b, c) WriteCommBlock(b, c)
 #  undef IN
 #  undef OUT
 #endif
@@ -202,20 +200,11 @@ static struct ConfigDialogData {
 	SMSWidgets sms;
 	UserWidget user;
 	GtkWidget *mailbox;
-	GtkWidget *help;
 } configDialogData;
 
 static gn_sms_message_center tempMessageSettings;
 
 
-static inline void Help1(GtkWidget * w, gpointer data)
-{
-	gchar *indx = g_strdup_printf("/help/%s/index.html", xgnokiiConfig.help_locale);
-	Help(w, indx);
-	g_free(indx);
-}
-
-
 void GUI_InitCallerGroupsInf(void)
 {
 	D_CallerGroup *cg;
@@ -689,10 +678,11 @@ void GUI_ShowOptions(void)
 
 	gtk_entry_set_text(GTK_ENTRY(configDialogData.connection.model), xgnokiiConfig.model);
 
-	gtk_entry_set_text(GTK_ENTRY(configDialogData.connection.init), xgnokiiConfig.initlength);
-
 	gtk_entry_set_text(GTK_ENTRY(configDialogData.connection.bindir), xgnokiiConfig.bindir);
 
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(configDialogData.connection.init),
+				  (gdouble) xgnokiiConfig.initlength);
+
 	if (xgnokiiConfig.connection == GN_CT_Irda) {
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(configDialogData.connection.irda),
 					     TRUE);
@@ -797,9 +787,6 @@ void GUI_ShowOptions(void)
 	} else
 		gtk_widget_hide(mail_option_frame);
 
-	/* Help */
-	gtk_entry_set_text(GTK_ENTRY(configDialogData.help), xgnokiiConfig.helpviewer);
-
 	optionsDialogIsOpened = TRUE;
 	gtk_widget_show(OptionsDialog);
 }
@@ -831,6 +818,7 @@ static void _MainExit(void)
 void MainExit(gchar *ermsg)
 {
 	if (ermsg) {
+		gn_log_xdebug(ermsg);
 		/* Nasty workaround -- that's some race condition that makes sometimes xgnokii to hang on exit */
 		sleep(1);
 		gtk_label_set_text(GTK_LABEL(infoDialog.text), ermsg);
@@ -904,12 +892,12 @@ static gint ButtonPressEvent(GtkWidget * widget, GdkEventButton * event)
 			    event->y >= 15 && event->y <= 30)) {
 			GUI_ShowContacts();
 		} else if (event->x >= 190 && event->x <= 210 && event->y >= 70 && event->y <= 85) {
-			if (!phoneMonitor.supported & PM_SMS)
+			if ((!phoneMonitor.supported) & PM_SMS)
 				phoneMonitor.working = _("SMS not supported!");
 			else
 				GUI_ShowSMS();
 		} else if (event->x >= 235 && event->x <= 248 && event->y >= 27 && event->y <= 75) {
-			if (!phoneMonitor.supported & PM_CALENDAR)
+			if ((!phoneMonitor.supported) & PM_CALENDAR)
 				phoneMonitor.working = _("Calendar not supported!");
 			else
 				GUI_ShowCalendar();
@@ -1011,10 +999,6 @@ static void OptionsApplyCallback(GtkWidget * widget, gpointer data)
 		xgnokiiConfig.mailbox =
 		    g_strdup(gtk_entry_get_text(GTK_ENTRY(configDialogData.mailbox)));
 	}
-
-	/* Help */
-	g_free(xgnokiiConfig.helpviewer);
-	xgnokiiConfig.helpviewer = g_strdup(gtk_entry_get_text(GTK_ENTRY(configDialogData.help)));
 }
 
 
@@ -1179,11 +1163,6 @@ static GtkWidget *CreateMenu(void)
 	gtk_menu_append(GTK_MENU(menu), menu_items);
 	gtk_widget_show(menu_items);
 
-	menu_items = gtk_menu_item_new_with_label(_("Help"));
-	gtk_menu_append(GTK_MENU(menu), menu_items);
-	gtk_signal_connect_object(GTK_OBJECT(menu_items), "activate", GTK_SIGNAL_FUNC(Help1), NULL);
-	gtk_widget_show(menu_items);
-
 	menu_items = gtk_menu_item_new_with_label(_("About"));
 	gtk_menu_append(GTK_MENU(menu), menu_items);
 	gtk_signal_connect_object(GTK_OBJECT(menu_items), "activate",
@@ -1668,20 +1647,6 @@ static GtkWidget *CreateOptionsDialog(void)
 	gtk_container_add(GTK_CONTAINER(vbox), hbox);
 	gtk_widget_show(hbox);
 
-	label = gtk_label_new(_("Init length:"));
-	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
-	gtk_widget_show(label);
-
-	configDialogData.connection.init = gtk_entry_new_with_max_length(100);
-	gtk_widget_set_usize(configDialogData.connection.init, 220, 22);
-	gtk_entry_set_editable(GTK_ENTRY(configDialogData.connection.init), FALSE);
-	gtk_box_pack_end(GTK_BOX(hbox), configDialogData.connection.init, FALSE, FALSE, 2);
-	gtk_widget_show(configDialogData.connection.init);
-
-	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_container_add(GTK_CONTAINER(vbox), hbox);
-	gtk_widget_show(hbox);
-
 	label = gtk_label_new(_("Bindir:"));
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
 	gtk_widget_show(label);
@@ -1696,6 +1661,21 @@ static GtkWidget *CreateOptionsDialog(void)
 	gtk_container_add(GTK_CONTAINER(vbox), hbox);
 	gtk_widget_show(hbox);
 
+	label = gtk_label_new(_("Init length:"));
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
+	gtk_widget_show(label);
+
+	adj = (GtkAdjustment *) gtk_adjustment_new(0.0, 0.0, 1000, 1.0, 10.0, 0.0);
+	configDialogData.connection.init = gtk_spin_button_new(adj, 0, 0);
+	gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(configDialogData.connection.init), TRUE);
+	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(configDialogData.connection.init), TRUE);
+	gtk_box_pack_start(GTK_BOX(hbox), configDialogData.connection.init, FALSE, FALSE, 2);
+	gtk_widget_show(configDialogData.connection.init);
+
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_container_add(GTK_CONTAINER(vbox), hbox);
+	gtk_widget_show(hbox);
+
 	label = gtk_label_new(_("Connection:"));
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
 	gtk_widget_show(label);
@@ -2154,32 +2134,6 @@ static GtkWidget *CreateOptionsDialog(void)
 	gtk_box_pack_end(GTK_BOX(hbox), configDialogData.mailbox, FALSE, FALSE, 2);
 	gtk_widget_show(configDialogData.mailbox);
 
-	/* Help */
-	frame = gtk_frame_new(_("Help viewer"));
-	gtk_widget_show(frame);
-
-	vbox = gtk_vbox_new(FALSE, 0);
-	gtk_container_add(GTK_CONTAINER(frame), vbox);
-	gtk_widget_show(vbox);
-
-	label = gtk_label_new(_("Help"));
-	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), frame, label);
-
-	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);
-	gtk_widget_show(hbox);
-
-	label = gtk_label_new(_("Viewer:"));
-	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
-	gtk_widget_show(label);
-
-	configDialogData.help = gtk_entry_new_with_max_length(HTMLVIEWER_LENGTH - 1);
-	gtk_widget_set_usize(configDialogData.help, 220, 22);
-
-	gtk_box_pack_end(GTK_BOX(hbox), configDialogData.help, FALSE, FALSE, 2);
-	gtk_widget_show(configDialogData.help);
-
-
 	optionsDialogIsOpened = FALSE;
 	return dialog;
 }
@@ -2189,12 +2143,12 @@ static void TopLevelWindow(void)
 {
 	GtkWidget *drawing_area;
 	GdkBitmap *mask;
-	GtkStyle *style;
-	GdkGC *gc;
 	struct sigaction act;
 
+#if GTK_CHECK_VERSION (2, 6, 0)
+	gtk_window_set_default_icon_name("phone");
+#endif
 	GUI_MainWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-	gtk_window_set_wmclass(GTK_WINDOW(GUI_MainWindow), "MainWindow", "Xgnokii");
 /*  gtk_window_set_decorated (GTK_WINDOW (GUI_MainWindow), GTK_FALSE); */
 	gtk_widget_realize(GUI_MainWindow);
 
@@ -2214,10 +2168,6 @@ static void TopLevelWindow(void)
 	    gdk_pixmap_create_from_xpm_d(GUI_MainWindow->window, &mask,
 					 &GUI_MainWindow->style->white, (gchar **) XPM_background);
 
-//  gdk_window_set_icon_name (GUI_MainWindow->window, "XXX");
-	style = gtk_widget_get_default_style();
-	gc = style->black_gc;
-
 	/* Create the drawing area */
 	drawing_area = gtk_drawing_area_new();
 
@@ -2287,7 +2237,6 @@ static void SplashScreen(void)
 	GdkPixmap *gdk_pixmap;
 	GdkBitmap *mask;
 	GtkStyle *style;
-	GdkGC *gc;
 
 	SplashWindow = gtk_window_new(GTK_WINDOW_POPUP);
 	gtk_widget_realize(SplashWindow);
@@ -2296,7 +2245,6 @@ static void SplashScreen(void)
 	gtk_window_position(GTK_WINDOW(SplashWindow), GTK_WIN_POS_CENTER);
 
 	style = gtk_widget_get_default_style();
-	gc = style->black_gc;
 	gdk_pixmap = gdk_pixmap_create_from_xpm_d(SplashWindow->window, &mask,
 						  &style->bg[GTK_STATE_NORMAL], XPM_logo);
 	pixmap = gtk_pixmap_new(gdk_pixmap, mask);
@@ -2324,28 +2272,6 @@ static gint RemoveSplash(GtkWidget * Win)
 	return FALSE;
 }
 
-static bool LocaleHelpExists(gchar *help_locale)
-{
-#ifdef WIN32
-	bool retval = true;
-#else
-	bool retval;
-	gchar *file;
-	struct stat buf;
-
-	if (!help_locale)
-		return false;
-
-	file = g_strdup_printf("/%s/help/%s", xgnokiiConfig.xgnokiidir, help_locale);
-	if (stat(file, &buf) == 0)
-		retval = true;
-	else
-		retval = false;
-	g_free(file);
-#endif
-	return retval;
-}
-
 static void ReadConfig(void)
 {
 	if (GN_ERR_NONE != gn_lib_phoneprofile_load( NULL, &statemachine ))
@@ -2353,8 +2279,8 @@ static void ReadConfig(void)
 
 	xgnokiiConfig.model = statemachine->config.model;
 	xgnokiiConfig.port = statemachine->config.port_device;
-	asprintf(&xgnokiiConfig.initlength, "%d", statemachine->config.init_length);
 	xgnokiiConfig.connection = statemachine->config.connection_type;
+	xgnokiiConfig.initlength = statemachine->config.init_length;
 	xgnokiiConfig.bindir = gn_lib_cfg_get("global", "bindir");
 	if (!xgnokiiConfig.bindir)
 		xgnokiiConfig.bindir = gn_lib_cfg_get("gnokiid", "bindir");
@@ -2379,30 +2305,6 @@ static void ReadConfig(void)
 	    xgnokiiConfig.callerGroups[2] = xgnokiiConfig.callerGroups[3] =
 	    xgnokiiConfig.callerGroups[4] = xgnokiiConfig.callerGroups[5] = NULL;
 	xgnokiiConfig.smsSets = 0;
-#ifdef WIN32
-	xgnokiiConfig.locale = "";
-	xgnokiiConfig.help_locale = "";
-#else
-	if ((xgnokiiConfig.locale = getenv("LC_ALL")) == NULL)
-		if ((xgnokiiConfig.locale = getenv("LC_MESSAGES")) == NULL)
-			if ((xgnokiiConfig.locale = getenv("LANG")) == NULL)
-				xgnokiiConfig.locale = "POSIX";
-	xgnokiiConfig.help_locale = g_strdup(xgnokiiConfig.locale);
-#endif
-	/* we have set en_US.UTF8 locale that is not supported by xgnokii */
-	if (!LocaleHelpExists(xgnokiiConfig.help_locale)) {
-		char *pos = strchr(xgnokiiConfig.locale, '.');
-		g_free(xgnokiiConfig.help_locale);
-		if (pos)
-			xgnokiiConfig.help_locale = g_strndup(xgnokiiConfig.locale,
-							      pos - xgnokiiConfig.locale);
-		else
-			xgnokiiConfig.help_locale = NULL;
-		if (!LocaleHelpExists(xgnokiiConfig.help_locale)) {
-			g_free(xgnokiiConfig.help_locale);
-			xgnokiiConfig.help_locale = g_strdup("en_US");
-		}
-	}
 }
 
 
