@@ -1,6 +1,6 @@
 /*
 
-  $Id: xgnokii_data.c,v 1.16 2007/12/04 19:27:37 hadess Exp $
+  $Id$
 
   X G N O K I I
 
@@ -44,13 +44,6 @@ bool GTerminateThread = false;
 bool enabled = false;
 static GtkWidget *label = NULL;
 
-static inline void Help1(GtkWidget * w, gpointer data)
-{
-	gchar *indx = g_strdup_printf("/help/%s/windows/data/index.html", xgnokiiConfig.help_locale);
-	Help(w, indx);
-	g_free(indx);
-}
-
 static void UpdateStatus(void)
 {
 	gchar *buf;
@@ -67,9 +60,9 @@ static void UpdateStatus(void)
 }
 
 
-inline void GUI_ShowData(void)
+void GUI_ShowData(void)
 {
-	if (!phoneMonitor.supported & PM_DATA)
+	if ((!phoneMonitor.supported) & PM_DATA)
 		return;
 
 	if (GTerminateThread) {
@@ -82,12 +75,6 @@ inline void GUI_ShowData(void)
 	gtk_window_present(GTK_WINDOW(GUI_DataWindow));
 }
 
-
-inline void GUI_HideData(void)
-{
-	gtk_widget_hide(GUI_DataWindow);
-}
-
 static inline void DisableData(GtkWidget * widget, gpointer data)
 {
 	gn_vm_terminate();
